###############################################################################################
# $Id: event_loop.py,v 1.16 2002/09/05 10:37:56 chakie Exp $
###############################################################################################

import sys
import scenario
import time
import string
import socket
import constants

import properties
import net.connection

import plan.factory
import action.factory
import immediate.factory

from select import select

# flag indicating wether orders have been done already
orders_done = 0

#################################################################################################
def crash ():
    """This function is called when we have a server crash. It prints some info and exits."""
    print "crash: server seems to have crashed, exiting"
    print "crash: this should save state or do something intelligent"
    sys.exit ( 1 )


#################################################################################################
def applyAction ():
    """Applies all the action that the server has sent us for the last turn. Normal clients would
    view this step by step and see what happened, but the AI client will just do all the updates
    immediately without worrying about how something happens. The internal structures will after
    this function be fully updated and reflect all the changes form the last turn."""

    # start from the first step. This is really not necessary, but is here more as a reminder
    scenario.time.firstStep ()

    # loop until we're worked over all the steps in the action
    while 1:
        # get current step
        step = scenario.time.getStep ()
        
        # do we have any action for this step?
        if scenario.events.has_key ( step ):
            # yes, get the action
            events = scenario.events [ step ]
        
            # loop over theme all and do the apply
            for event in events:
                # apply it
                print "applyAction: should apply", event

        # advance the step one further if possible
        if not scenario.time.nextStep ():
            # could not advance any more, all action is now applied
            return
        

#################################################################################################
def doTurnUpdates ():
    """Function for performing all needed updates for a new turn. Advances the current turn and
    updates all received events. All events that are in the global listhash of incoming processed and
    dispalyable events for this turn are retrieved and updated to the internal data."""
 
    # optimizations for speed. Cache locally
    turn   = scenario.time.getTurn ()
    events = scenario.events 

    # no commands for this turn an no immediate either
    commands = []
    immediate = []

    # do we have any commands for this turn?
    if events.has_key ( turn ):
        # get the commands
        commands = events [ turn ]

        # delete them
        del events [ turn ]

    # do we have any immediate commands?
    if events.has_key ( -1 ):
        # get the commands
        immediate = events [ -1 ]

        # delete them
        del events [ -1 ]

    # loop over them all
    for command in commands + immediate:
        # update the command now
        print "doTurnUpdates: applying command: %s" % str( command ).strip ()
        
        command.applyAI ()

 
#################################################################################################
def handleUpdate (update):
    """Handles an immediate update that has arrived on the network. The update is sent to the
    currenttly active game state for handling."""

    global orders_done

    #print "handleUpdate: handling ", update.toString().strip ()

    # execute it in case it has something meaningful
    update.execute ()

    # see wether we have started an action phase or orders phase
    if update.getName () == "endaction":
        # all action has ended and is ready for display, update the action
        applyAction ()

        # orders for this new orders phase are not yet done
        orders_done = 0

    elif update.getName () == "endturn":
        # orders ended, nothing really we need to do here, we wait for all stuff to arrive from the
        # server and then a 'endaction' means that all action has arrived
        pass

        
#################################################################################################
def checkNetEvents (may_sleep_seconds):
    """Checks wether there are any network events ready to be received. If there is something to be
    handled it is done. Handled means that a line of data is read from the connection and a new
    command is"""

    # get a line of data if one is availabel
    try:
        line = scenario.connection.readLine ()
    except:
        # failed to read from the socket, remote host closed down?
        crash ()
        
    # did we get anything?
    if line:
        #print "checkNetEvents: got '%s'" % line

        # split the data
        parts = line.split ()

        # did we get any data at all?
        if len ( parts ) == 0:
            # oops, nothing there?
            raise "checkNetEvents: no parameters at all?"
       
        # have the plan factory try to make an immediate update
        update = immediate.factory.create ( parts )

        # did we get an update?
        if update != None:
            # yes, it was an update, handle it
            handleUpdate ( update )
            return

        
        # have the action factory try to make an action instance
        act = action.factory.create ( parts )

        # did we get an update?
        if act != None:
            # yes, it was an update, handle it
            act.execute ()
            return
        
        # no update, have the plan factory make a plan
        newplan = plan.factory.create ( parts )

        #print "checkNetEvents: got '%s'" % newplan.toString ().strip ()

    else:
        # Too small values means we don't sleep
        # Note that "== 0" might be bad because this is floating point
        if may_sleep_seconds < 0.01:
            return

        incoming, out, exceptional  = select ( [scenario.connection ], [], [], may_sleep_seconds)
        # Did we get anything?
        if len(incoming) == 0:
            return
        
        # We got something, so call us again.
        # Don't sleep again, tho
        checkNetEvents(0.0)
        

#################################################################################################
def sendPlans ():
    """Sends all plans that the units have to the server. As we're the client we need to send
    the number of plans as an immediate update: 'end_turn' so that the server knows how many
    plans to expect""" 

    # all plans
    plans = []
        
    # loop over all units in the map
    for unit in scenario.info.units.values ():
        # is the unit visible
        if unit.getOwner () != scenario.local_player_id:
            # not our unit, get next
            continue

        # add the unit plans
        plans += unit.getPlans ()

    # now loop over all plans and send them
    for plan in plans:
        scenario.connection.send ( plan.toString () )
  
    print "sendPlans: sent %d plans" % len (plans)


#################################################################################################
def updateAI ():
    """This is the main point where the AI should do its work. It gets called very often, i.e. after
    every time that network events have been checked. It should therefore maybe not every call do
    full heavy calculations. Anyway, this is the place."""
    global orders_done
    
    # do stuff
    print "updateAI: should perform AI calculations"

    # orders are now given for this orders phase
    orders_done = 1

    # send off all plans first
    sendPlans ()
    
    # create a new plan that says how many plans the server should expect
    update = immediate.end_turn.EndTurn ( scenario.time.getTurn () )
        
    # send off the plan
    scenario.connection.send ( update.toString () )
 

    
#################################################################################################
def eventLoop ():
    """Main function that handles the entire event loop. Polls events from the SDL event system and
    handles them. Regularly checks wether there is any incoming data to be read."""

    global orders_done

    # How many seconds we may select() on the socket
    may_sleep_seconds = 0.0

    # loop forever, or at least while we're playing
    while scenario.playing != constants.GAME_ENDED:
        checkNetEvents(may_sleep_seconds)
        
        # do ai stuff if not already done. orders will be given only once for each orders phase
        if not orders_done:
            updateAI ()
        else:
            may_sleep_seconds = 1.0
        
    print "event_loop: main loop terminated, exiting."
    
    
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
