###############################################################################################
# $Id: audio_manager.py,v 1.9 2002/04/17 08:50:50 chakie Exp $
###############################################################################################

import properties
import properties_audio
import pygame

class AudioManager:
    """
    This class is the central handler for all audio in Civil. It takes care of:

    * initializing the audio subsystem
    * loading samples from disk
    * loading music from disk
    * playing samples and music
    * pausing the music

    * checking for an audio CD
    * playing, stopping and pausing audio CD playing
    * ejecting the CD tray

    If no audio is available all operations are dummy operations and samples are just never
    played. Thus the 'clients' of this interface need never worry about wether audio is avilable or
    not, they just play samples and music without worrying about that.
    """
    

    def __init__ (self):
        """Initializes the audio manager. Checks wether audio can be used at all, and if it can
        determines and sets up parameters."""

        # try to init audio
        try:
            # if this succeeds we assume audio is present, otherwise not
            pygame.mixer.init ()

            # get the number of channels
            self.channels = pygame.mixer.get_num_channels ()

            # we do have audio but no music
            self.audio      = 1
            self.music      = 0
            self.playing    = 0
            self.paused     = 0
            self.paused_cd  = 0

            # no samples loaded yet
            self.loadedsamples = {}
            
        except:
            # no channels and no audio
            self.channels   = 0
            self.audio      = 0
            self.music      = 0
            self.playing    = 0
            self.playing_cd = 0
            self.paused     = 0
            self.paused_cd  = 0

        # try to init CD audio
        try:
            # if this succeeds we assume a CD with audio is present in the cdrom drive, otherwise not
            pygame.cdrom.init ()

            # check for an audio cd
            if not self.checkForAudioCD ():
                # no audio cd
                raise "not an audio cd"
 
            self.audio_cd   = 1
            self.playing_cd = 0
            self.paused_cd  = 0
            self.track      = 0
            
        except:
            # no cd or no cd with audio
            self.audio_cd   = 0
            self.playing_cd = 0
            self.paused_cd  = 0
            self.track      = 0
         
        # print some info
        if self.audio:
            print "audio: %d sound channels available" % self.channels
        else:
            print "audio: no audio detected"

        if self.audio_cd:
            print "audio: found audio CD with %d tracks" % self.cd.get_numtracks() 
        else:
            print "audio: no audio CD found"
           

    def getNumChannels (self):
        """Returns the number of audio channels available. This is 0 if no audio is available at
        all. """
        return self.available


    def disableAudio (self):
        """Disables audio even if audio is available. This can be used for instance if a commandline
        option has been given."""
        self.audio = 0
        

    def hasAudio (self):
        """Returns 1 if audio is available and 0 if not."""
        return self.audio


    def playSample (self, sample):
        """Plays a sample. If no audio is available this method does nothing. If the sample is not
        already loaded then it will be loaded, stored and then played."""

        # do we have audio at all?
        if not self.audio:
            # no, go away
            return

        # do we already have the sample loaded
        if not self.loadedsamples.has_key ( sample ):
            # nope, get the filename for that logical name
            try:
                name = properties_audio.samplenames [ sample ]
            except KeyError:
                # no such sample
                raise "AudioManager.playSample: no such sample: '%s'" % sample
            
            # all ok so far, load the sample
            sampledata = pygame.mixer.Sound ( name )

            # store it in our cache
            self.loadedsamples [sample] = sampledata

        else:
            # it's loaded, just get it
            sampledata = self.loadedsamples [sample]
            
        # play the sample
        sampledata.play ()


    def loadMusic (self, music):
        """Loads 'music' as the background music. Does not yet start playing it. Only one music can
        be loaded at one time, so calling this method repeatedly overwrites the current
        music. Nothing is returned, as the music is stored internally. If audio is not enabled this
        method does nothing."""
        
        # do we have audio or musicat all?
        if not self.audio:
            # no, go away
            return

        # load it
        try:
            pygame.mixer.music.load ( music )

            # now we have music
            self.music = 1

        except:
            # failed?
            raise
            raise "AudioManager.loadMusic: failed to load: " + music


    def playMusic (self):
        """Plays the current music. If audio is not enabled this method does nothing. If audio is
        enabled but music is not loaded using the method 'loadMusci()' then a warning is printed."""

        # do we have audio or musicat all?
        if not self.audio:
            # no, go away
            return

        # do we have music loaded?
        if not self.music:
            # no music?
            print "AudioManager.playMusic: no music loaded, can't play"
            return

        # play music
        pygame.mixer.music.play ()

        # we're playing now
        self.playing = 1


    def stopMusic (self):
        """Stops playing music if it is playing. Does nothing if no mucis is playing or if there is
        no audio available."""

        # do we have audio or music at all?
        if not self.audio or not self.music:
            # no, go away
            return

        # stop music
        pygame.mixer.music.stop ()

        # we're stopped now
        self.playing = 0


    def pauseMusic (self):
        """Pauses playing music if it is playing. Does nothing if no music is playing or if there is
        no audio available."""

        # do we have audio or music at all?
        if not self.audio or not self.music:
            # no, go away
            return

        # are we paused?
        if self.paused:
            # yep, go away
            return
        
        # pause music
        pygame.mixer.music.pause ()

        # we're paused now
        self.paused = 1
        
            
    def unpauseMusic (self):
        """Resumes playing music if it is paused. Does nothing if no music is already playing or if
        there is no audio available."""

        # do we have audio or music at all?
        if not self.audio or not self.music:
            # no, go away
            return

        # are we not paused?
        if not self.paused:
            # nope, go away
            return
 
        # pause music
        pygame.mixer.music.unpause ()

        # we're not paused anymore
        self.paused = 0


    def fadeoutMusic (self, milliseconds):
        """Fades out the music to silent over a period of 'milliseconds'. If no music is playing or
        no audio is used this method does nothing."""

        # do we have audio or music at all?
        if not self.audio or not self.music or not self.playing:
            # no, go away
            return
        
        # fade out the music
        pygame.mixer.music.fadeout ( milliseconds )
        

    def isPlaying (self):
        """Returns 1 if we're currently playing music. It returns 1 even if we're currently
        paused. It mainly means that 'playMusic()' was called later than 'stopMusic()'. Returns 0 if
        music is not playing."""
        return self.playing


    def isPaused (self):
        """Returns 1 if we're currently playing music but paused.  Returns 0 if not paused."""
        return self.paused




    def checkForAudioCD (self):
        """Checks the drive for an audio CD. Returns 1 if found and None of not. Stores the CD
        object internally if ok."""

        # precautions
        if pygame.cdrom.get_count () == 0:
            # no cdrom
            return 0

        # init the cd by creating a CD class and initializeing it
        self.cd = pygame.cdrom.CD ( 0 )
        self.cd.init () 

        # does it have any tracks at all?
        if self.cd.get_numtracks () == 0:
            # no tracks, can't play
            return 0
        
        # loop over all tracks
        for index in range ( self.cd.get_numtracks () ):
            # is this an audio track?
            if not self.cd.get_track_audio ( index ):
                # not an audio track, not an audio cd
                return 0

        # we got this far, so it has to be an audio cd, as all tracks are audio, and there is at
        # least one track
        return 1
    

    def hasAudioCD (self):
        """Returns 1 if an audio CD is available and 0 if not."""
        return self.audio_cd


    
    def isPlayingCD (self):
        """Returns 1 if we're currently playing music from an audio CD. It returns 1 even if we're
        currently paused. It mainly means that 'playMusicCD()' was called later than
        'stopMusicCD()'. Returns 0 if CD music is not playing."""
        return self.playing_cd


    def playCD (self):
        """Starts playing the CD from the current track. If no audio CD has been found or we are
        already playing a CD nothing will be done."""
        # do we have an audio cd available or are we already plyaing?
        if not self.audio_cd or self.playing_cd:
            # no, go away
            return

        # start playing the current track
        self.cd.play ( self.track )

        # we're playing now
        self.playing_cd = 1
        

    def stopCD (self):
        """Stops playing an audio CD if one is available and we are currently playing one."""
        # do we have an audio cd available or are we alreayd stopped?
        if not self.audio_cd or not self.playing_cd:
            # no, go away
            return

        # stop playing
        self.cd.stop ()

        # we're not playing now
        self.playing_cd = 0
       

    def pauseCD (self):
        """ """
        # do we have an audio cd available?
        if not self.audio_cd:
            # no, go away
            return

 
    def unpauseCD (self):
        """ """
        # do we have an audio cd available?
        if not self.audio_cd:
            # no, go away
            return


    def nextTrackCD (self):
        """Starts playing the next track of the cd. Stops playing if we've come to the end of the
        cd."""  
        # do we have an audio cd available?
        if not self.audio_cd:
            # no, go away
            return

        # nexttrack
        self.track += 1

        # too far?
        if self.track == self.cd.get_numtracks ():
            # yep, stop playing
            self.stopCD ()

            # and let the last track be the max
            self.track -= 1
            


    def prevTrackCD (self):
        """Starts playing the previous track of the cd. Starts from track 0 if we've come to the
        beginning of the cd.""" 
        # do we have an audio cd available?
        if not self.audio_cd:
            # no, go away
            return

        # decrement the track
        self.track -= 1

        # too far?
        if self.track == -1:
            # yep, goto the first tracj
            self.track = 0
            

    def ejectCD (self):
        """Ejects the CD tray if possible. If we are currently playing CD music it is of course
        stopped first."""
        # are we playing and audio cd right now?
        if self.playing_cd:
            # yes, stop playing
            self.stopCD ()
    
        # eject the cd
        self.cd.eject ()
   
        
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
