###############################################################################################
# $Id: assign_targets.py,v 1.14 2002/09/04 16:44:34 msa Exp $
###############################################################################################

import scenario

from engine.ai.module       import Module
from unit                   import HOLDFIRE, DEFENSIVEFIRE, FIREATWILL
from action.set_mode_act    import SetModeAct
from action.set_target_act  import SetTargetAct
from action.add_plan_act    import AddPlanAct
from plan.skirmish          import Skirmish


class AssignTargets (Module):
    """This class defines a module that when executed checks units that have no target assigned to
    them and assigns a skirmish target. This reduces the click-load for the players, as the server
    will aid in giving the units targets.

    The code will favour the closest unit that is accessible without rotating, but it will rotate
    the unit if no other target is accessible.

    This module is executed every step.
    """


    def __init__ (self):
        """Initializes the module. Sets a suitable name."""

        # call superclass with the delays
        Module.__init__ (self, 'assign targets' )


    def isReady (self, step):
        """Checks wether the module is ready to be executed. This module is always ready, so it
        always returns 1."""
        # always ready!
        return 1
    

    def execute (self, step):
        """Executes the module. Loops over all units and assigns suitable skirmish targets to units
        that have no current target."""

        # no action yet
        actiondata = []
        
        # loop over all units we have
        for unit in scenario.info.units.values ():
            # does the unit already have some other plan?
            if len ( unit.getPlans () ) > 0:
                # unit already has plans, so not eligible for skirmish
                continue

            # does it already have a target? no use to assign many targets
            if unit.getTarget () != None:
                # already has a target
                continue

            # can the unit skirmish in its current mode?
            if not unit.getMode ().canSkirmish ():
                # unit can't skirmish in it's current mode
                continue

            # get the combat policy
            policy = unit.getCombatPolicy ()
            
            # is the unit not allowed to do anything?
            if policy == HOLDFIRE:
                # do nothing with this unit
                continue

            # check morale, fatigue
            if unit.getMorale ().checkSkirmish () == 0 or unit.getFatigue ().checkSkirmish () == 0:
                # either the morale is too low or the fatigue is too high, we won't do combat
                print "AssignTargets.execute: morale or fatigue fails, no target for:", unit
                continue
            
            if policy == DEFENSIVEFIRE:
                # defensive fire only, check for a suitable attacker
                self.checkDefensiveFire ( unit )
                continue
            
            # unit is allowed to fire at will

            # store the unit owner for speed
            owner = unit.getOwner ()
            mindistance = 10000000

            # no chosen target yet
            chosentarget = None
            
            # unit is eligible for skirmish, so find a unit that it should target
            for potentialtarget in scenario.info.units.values ():
                # is it a unit of the opposite side
                if potentialtarget.getOwner () == owner:
                    # same side, no target
                    continue
                    
                # it can be a potential enemy. do we see it and is it in range?
                if not unit.inRange ( potentialtarget ) or not unit.seesEnemy ( potentialtarget ):
                    # not in range or not visible
                    continue

                # get the distance to the unit. The distance is squared, so it's just for comparison
                distance = unit.distance ( target=potentialtarget, simple=1 )

                # add to the distance if we're not facing the enemy, and thus prefer those that are
                # available without rotating
                if not unit.pointedAt ( potentialtarget ):
                    # double the distance
                    distance *= 2

                # is the distance less than the currently closest distance?
                if distance < mindistance:
                    # yep, we have a new target candicate
                    mindistance = distance
                    chosentarget = potentialtarget

            # now we've gone through all units, do we have a chosen target?
            if chosentarget != None:
                # yep, the unit now has a suitable target, create a new 'skirmish' plan and assign it
                plan = Skirmish ( oldmode=unit.getMode (), unitid=unit.getId (), targetid=chosentarget.getId () )

                # add it to the plans for the unit
                unit.getPlans ().append ( plan )

                actiondata.append ( AddPlanAct ( step, unit.getId (), plan ) )
                    
                #create an action for the new target 
 ##                actiondata.append ( SetTargetAct ( step, unit.getId (), chosentarget.getId () ))

##                 # we should also set the mode for the unit to a skirmishing mode
##                 actiondata.append ( SetModeAct ( step, unit.getId (), unit.getMode ().onSkirmish () ))

##                 # set the target for the unit too
##                 unit.setTarget ( chosentarget )
                
##                 # create the proper skrimish mode instance and assign it to the unit
##                 unit.setMode ( createMode ( unit.getMode ().onSkirmish () ) )

                print "AssignTargets.execute: %s skirmish with %s" % (unit.getName (),
                                                                      chosentarget.getName())

        # we're done with all the assigning of targets, return all action we got
        return actiondata
    

    def checkDefensiveFire (self, unit):
        """Checks wether there are any suitable enemies that are attacking this unit. If there is
        and they are in range then they are set as targets for the unit."""

        print "AssignTargets.checkDefensiveFire: do nada for %d" % unit.getId ()
        # store the unit owner for speed
        owner = unit.getOwner ()
        mindistance = 10000000

        # no chosen target yet
        chosentarget = None
            
        # unit is eligible for skirmish, so find a unit that it should target
        for potentialtarget in scenario.info.units.values ():
            # is it a unit of the opposite side
            if potentialtarget.getOwner () == owner:
                # same side, no target
                continue

            # des the enemy currently have our unit as a target?
            if potentialtarget.getTarget () != unit:
                # no, so it can't be fired upon either
                continue

            # get the distance to the unit. The distance is squared, so it's just for comparison
            distance = unit.distance ( target=potentialtarget, simple=1 )

            # add to the distance if we're not facing the enemy, and thus prefer those that are
            # available without rotating
            if not unit.pointedAt ( potentialtarget ):
                # double the distance
                distance *= 2

            # is the distance less than the currently closest distance?
            if distance < mindistance:
                # yep, we have a new target candicate
                mindistance = distance
                chosentarget = potentialtarget

        # TODO: this is broken! still old realtime code!

        # now we've gone through all units, do we have a chosen target?
        if chosentarget != None:
            # yep, create a new skirmish plan
            plan = Skirmish ( unitid = unit.getId (), turn = scenario.time.getTurn(),
                              targetid = chosentarget.getId () )

            # attach the plan to the server (and send out to the clients)
            plan.attachServer ( engine.getOutgoingQueue () )

            print "AssignTargets.checkDefensiveFire: %s skirmish with %s" % (unit.getName (),
                                                                             chosentarget.getName())
        
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
