###############################################################################################
# $Id: module.py,v 1.8 2002/09/07 19:08:33 chakie Exp $
###############################################################################################

from action.set_mode_act         import SetModeAct
from action.change_modifiers_act import ChangeModifiersAct
from action.set_target_act       import SetTargetAct
from mode.mode_factory           import createMode

class Module:
    """This class defines the interface for module classes. A module is simply some code that plugs
    into the engine and provides simple AI-like functionality. Modules are executed by the engine
    every turn by running the method execute(). If a module should not be executed each turn it must
    keep track of that internally.

    Subclasses must override the method execute() and do something meaningful in it. They are also
    required to pass a name to the constructor of this class. The name can be used for debugging
    later.

    The method execute() should return a list of action data that the module created, and also to
    make sure that the same action is applied to the units.

    Every modules must implement the method 'isReady()' to indicate wether the meodule wants to be
    executed on a given step.
    """


    def __init__ (self, name):
        """Initializes the module. Stores the passed name."""
        self.name = name


    def getName (self):
        """Returns the name of the module."""
        return self.name
    

    def isReady (self, step):
        """Checks wether the module is ready to be executed. A module that is ready should return 1
        in order to be executed, and 0 if not. The parameter 'step' is the current step."""

        raise NotImplementedError ( "Module.isReady: this method must be overridden")


    def execute (self, step):
        """Executes the module. The module can perform whatever automatic tasks it wants to. This
        method must be overridden by sublasses.

        This should return a list of action data that the method created, and also to make sure that
        the same action is applied to the units.
        """
        
        raise NotImplementedError ( "Module.execute: this method must be overridden")



    def setMode (self, unit, mode, actiondata, step):
        """Convenience method creating a new mode for a unit and also create the needed action
        data."""
        # make action for the new mode and add to the result
        actiondata.append ( SetModeAct ( step, unit.getId (), mode ))
               
        # create the proper mode and assign it to the unit
        unit.setMode ( createMode ( mode ) )


    def clearTarget (self, unit, actiondata, step):
        """Clears the target for the unit. Modifies the unit and also create action for the
        clearing and puts it onto 'actiondata'. The unit is set to have whatever mode is proper for
        it when the current mode ends. """

        # clear the target for the unit 
        unit.setTarget ( None )

        # create action too
        actiondata.append ( SetTargetAct ( step, unit.getId (), -1 ))

        # set a mode too that should be when the unit is done with combat
        self.setMode ( unit, unit.getMode ().onDone (), actiondata, step )

 
    def changeModifiers (self, unit, actiondata, step):
        """This method is a convenience method for subclasses that need to generate action for
        changed modifiers. The call to make the action is a bit long and unwieldy, so it has been
        put here instead. Puts a new ChangeModifiersAct to the actiondata."""

        # create the action
        actiondata.append ( ChangeModifiersAct ( unitid=unit.getId (), step=step,
                                                 fatigue=unit.getFatigue ().getValue (),
                                                 morale=unit.getMorale ().getValue (),
                                                 experience=unit.getExperience ().getValue () ) )
    

         
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
