###############################################################################################
# $Id: resolve_combat.py,v 1.20 2002/06/10 10:13:43 chakie Exp $
###############################################################################################

import random
from math import log
import scenario

from engine.ai.module            import Module
from engine.executor.rotate_util import isFlankAttack
from action.damage_act           import DamageAct, KILLED, GUNSDESTROYED
from action.change_modifiers_act import ChangeModifiersAct
from action.set_target_act       import SetTargetAct


class ResolveCombat (Module):
    """This class defines a module that when executed checks all units that have a target and wether
    it can fire at this step. If the unit can fire then a round of firing will be executed and the
    necessary action sent out.

    A unit will not fire every step, even if it has a target, instead the firing speed depends on
    the unit skill and morale.
    
    This module is executed every step.
    """


    def __init__ (self):
        """Initializes the module. Sets a suitable name."""

        # call superclass
        Module.__init__ (self, 'resolve combat' )

        # a map of delays for each unit with a target
        self.delays = {}
        

    def isReady (self, step):
        """Checks wether the module is ready to be executed. This module is always ready, so it
        always returns 1."""
        # always ready!
        return 1
    

    def execute (self, step):
        """Executes the module. Loops over all units and checks wether they have targets and are
        available for firing. Units that can fire will fire one round. Returns the action generated
        from the firing."""

        # no action yet
        actiondata = []
        
        # loop over all units we have
        for attacker, target in scenario.engine.getCombatSchedule ():
            # does the attacker target still exist?
            if attacker.isDestroyed ():
                # attacker has died, nothing to do
                continue

            # target still ok?
            if target.isDestroyed ():
                # the target has been destroyed somehow, clear the target
                actiondata.append ( SetTargetAct ( step, attacker.getId (), -1 ))

                # clear the target for the unit 
                attacker.setTarget ( None )

                continue

            # resolve this attack
            self.resolveAttack ( attacker, target, actiondata, step )
            
        # we're done with the clearing of targets, return all action we got
        return actiondata


    def resolveAttack (self, attacker, target, actiondata, step):
        """Resolves one single attack where 'attacker' attacks 'target'. All needed action is added
        to 'actiondata'. The damage to the attacker and defender is directly applied to the units."""

        # no damage to anybody yet
        killed        = 0
        gunsdestroyed = 0

        # is the attacker within range for melee combat? if so we don't resolve it here, that has a
        # separate module
        if attacker.getMode ().isMeleeing ():
            # yep, the attacker is already meleeing, so we don't resolve that here
            print "ResolveCombat.resolveAttack: attacker already meleeing, not handled here"
            return

        print "ResolveCombat.resolveAttack: %s -> %s" % (attacker.getName (),target.getName ())

        # is the attacker an artillery unit with guns left?
        if attacker.getWeapon ().isArtillery () and attacker.getWeaponCounts ()[0] > 0:
            # yes, check for exploded gun. possibility: 2%
            if random.randrange ( 0, 100 ) < 2:
                # oops, boom, there goes a gun , create action for that
                actiondata.append ( DamageAct ( unitid=attacker.id, damagetype=GUNSDESTROYED,
                                                damagecount=1, step=step ) )

                # also apply the damage to the unit. one less intact gun, one more destroyed gun
                ok, destroyed = attacker.getWeaponCounts ()
                attacker.setWeaponCounts ( ok - 1, destroyed + 1)
 
        # calculate the full attack value
 
        # get weapon base damage of the unit
        value = attacker.getDamage ()

        # modify by attacker terrain
        value = value * scenario.map.getTerrain ( attacker.getPosition () ).attackModifier ()

        # modify by attacked mode
        value = value * 1.0
        
        # modify by attacker skill
        value = attacker.getExperience ().evaluateAttack ( value )
            
        # modify by attacker fatigue
        value = attacker.getFatigue ().evaluateAttack ( value )


        # now modify by defender data

        # modify by defender terrain. first get the terrain modifier and entrenchment level
        defensemod   = scenario.map.getTerrain ( target.getPosition () ).defenseModifier ()
        entrenchment = 1.0

        # modify value by defender terrain and entrenchment level
        value = value * defensemod * entrenchment
            
        # modify by attack direction. we use a deviation of 5 from the target facing to be the max
        # value before the attack is a flank attack. so the interval is +-5 from the target facing
        if isFlankAttack ( attacker, target, 5 ):
            # we have a flank attack, modify damage
            value = value * 1.5

        # modify by distance. At max distance the effectiveness is  greatly reduced.
        value = value * 1.0

        # modify by defender mode
        value = value * 1.0
            
        # modify by random value. range: 0.8 to 1.2
        value = value * random.randrange ( 80, 120 ) / 100

        # convert the value to a number of men killed and wounded. why log()?  um.... it seems to
        # normalize the ranges a bit???
        try:
            killed = log (value)
        except OverflowError:
            # hmm...
            killed = 0
        except ValueError:
            # hmm again?
            raise "ValueError raised for: %f" % value
            
        # if the defender has guns there may be a gun knocked out. possibility: 5%
        if target.getWeapon ().isArtillery () and random.randrange ( 0, 100 ) < 5:
            # a gun was destroyed
            gunsdestroyed = 1
        else:
            # no guns were destroyed
            gunsdestroyed = 0
                
        # modify morale and check for retreats, routs or gets disorganizations

        # check if attacker gets a morale boost

        # modify fatigue. the defender gets some minor added fatigue due to combat stress
        target.getFatigue ().setValue ( target.getFatigue ().getValue () + 1 )

        # modify experience

        # send out a damage command.  Damage command needs work...
        # also, we're currently ignoring destroyed guns.
        print "ResolveCombat.resolveAttack: unit: %d, killed: %d of %d" % (target.id, killed, target.men)
        
        # do we have destroyed guns?
        if gunsdestroyed > 0:
            # yep, add action for that too
            actiondata.append ( DamageAct ( unitid=target.id, damagetype=GUNSDESTROYED,
                                            damagecount=gunsdestroyed, step=step ) )

            # apply the damage. if the unit is destroyed it will mark itself as destroyed and remove
            # itself from the global unit structures
            target.applyDamage (type=GUNSDESTROYED, count=gunsdestroyed )
 

        # create action for the total damage
        actiondata.append ( DamageAct (unitid=target.id, damagetype=KILLED, damagecount=killed, step=step ))

        # apply the damage. if the unit is destroyed it will mark itself as destroyed and remove
        # itself from the global unit structures
        target.applyDamage ( type=KILLED, count=killed)

        # now add action for the optionalyl changed modifiers for both the attacker and
        # target. these all have already been applied locally, so we just copy the values from the
        # units and send 'em
        actiondata.extend ( self.createModifierActions ( attacker, target, step ) )
                            

    def createModifierActions (self, attacker, target, step):
        """Creates the needed action for sending out changed modifiers for the attacker and
        target. The target data is sent only if the target still is alive. The data is returned as
        one or two ChangeModifiersAct instances in a list."""

        # attacker is assumed to be alive
        act1 = ChangeModifiersAct ( unitid=attacker.id, step=step,
                                    fatigue=attacker.getFatigue ().getValue (),
                                    morale=attacker.getMorale ().getValue (),
                                    experience=attacker.getExperience ().getValue () )

        # is the target still alive? if it isn't we don't bother sending out action for that
        if target.isDestroyed ():
            # destroyed, we're done here
            return [ act1 ]

        # seems to be alive, so create action for it too
        act2 = ChangeModifiersAct ( unitid=target.id, step=step,
                                    fatigue=target.getFatigue ().getValue (),
                                    morale=target.getMorale ().getValue (),
                                    experience=target.getExperience ().getValue () )

        # and add them for sending out too
        return [act1, act2]

        
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
