###############################################################################################
# $Id: schedule_combat.py,v 1.5 2002/09/04 16:44:34 msa Exp $
###############################################################################################

import scenario

from engine.ai.module       import Module
from action.set_mode_act    import SetModeAct
from action.set_target_act  import SetTargetAct


class ScheduleCombat (Module):
    """This class defines a module that when executed checks all units that have a target and wether
    it can fire at this step. If the unit can fire then a round of firing will be executed and the
    necessary action sent out.

    A unit will not fire every step, even if it has a target, instead the firing speed depends on
    the unit skill and morale.
    
    This module is executed every step.
    """


    def __init__ (self):
        """Initializes the module. Sets a suitable name."""

        # call superclass
        Module.__init__ (self, 'schedule combat' )

        # a map of delays for each unit with a target
        self.delays = {}
        

    def isReady (self, step):
        """Checks wether the module is ready to be executed. This module is always ready, so it
        always returns 1."""
        # always ready!
        return 1
    

    def execute (self, step):
        """Executes the module. Loops over all units and checks wether they have targets and are
        available for firing. Units that can fire will fire one round. Returns the action generated
        from the firing."""

        # no action yet
        actiondata = []
        
        # loop over all units we have
        for unit in scenario.info.units.values ():
            # does the unit already have a target?
            target = unit.getTarget ()
            if target == None:
                # no target, make sure it's not in ar map of delays. when it aquires a target it
                # will be reinserted
                if self.delays.has_key ( unit.getId () ):
                    del self.delays [unit.getId ()]

                # next unit
                continue

            # TODO: this relies on that the units always have damage applied to them, as the next
            # check needs to know if a unit is already destroyed.

            # does the target still exist?
            if target.isDestroyed ():
                # the target has been destroyed somehow, clear the target
                actiondata.append ( SetTargetAct ( step, unit.getId (), -1 ))

                # clear the target for the unit 
                unit.setTarget ( None )
                continue

            # seems the target is still ok. does the unit already have an allocated delay?
            if not self.delays.has_key ( unit.getId () ):
                # no, it doesn't, so it must be assigned
                self.delays [ unit.getId () ] = 10
                continue
            
            # decrement the firing delay for the unit
            delay = self.delays [ unit.getId () ] - 1
            self.delays [ unit.getId () ] = delay
           
            # have we reached 0, i.e. no delay left?
            if delay > 0:
                # no combat yet
                continue

            
            # it's time for some combat, reset a new delay

            # TODO: this value is fake!
            self.delays [ unit.getId () ] = 10

            # this unit will now fire a round of fire, but it'll be resolved by another module. why
            # this hassle then? well, we want to concentrate all combat resolution into one place,
            # and as other executors will also generate firing events, there is a separate structure
            # with that data, and a separate module too. It could all be here just as well, but /me
            # likes the code simple and clean
            scenario.engine.getCombatSchedule ().append ( (unit, target) )
            
            
        # we're done with the clearing of targets, return all action we got
        return actiondata

        
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
