###############################################################################################
# $Id: units_destroyed.py,v 1.10 2002/09/05 10:37:57 chakie Exp $
###############################################################################################

import scenario
import server.properties
import constants
from organization           import REBEL, UNION
from engine.ai.module       import Module

from action.end_game_act    import EndGameAct


class UnitsDestroyed (Module):
    """This class defines a module that when executed checks wether one or both of the armies have
    lost more units than a specified percentage, say 75%. If this happens then the army is
    considered to be collapsed and not useful anymore and the server will automatically surrender
    that player. If both armies have been reduced sufficiently then a draw will be declared, as
    neither army is anymore fit for combat.

    The setting module_unitsdestroyed_percantage determines the minimum morale.
    
    This module is executed once every minute.

    TODO: handle victories and draws somehow
    """


    def __init__ (self):
        """Initializes the module. Sets a suitable name."""

        # call superclass with the delays
        Module.__init__ (self, 'unitsdestroyed' )


    def isReady (self, step):
        """Checks wether the module is ready to be executed. This module is ready on the last step
        of the action."""

        #is this the last turn?
        if step == scenario.time.getMaxStep ():
            # we're ready now!
            return 1

        # not yet last turn
        return 0


    def execute (self, step):
        """Executes the module. Counts the number of units that are left for both armies, and if
        either (or both) players have less units left than a certain % then the game is ended."""
       
        # nothing alive at all yet
        ok        = { REBEL: 0, UNION: 0 }
        destroyed = { REBEL: 0, UNION: 0 }
        
        # loop over all ok units
        for unit in scenario.info.units.values ():
            # add to the counts
            ok [unit.getOwner ()] += 1

        # loop over all destroyed units
        for unit in scenario.info.destroyed_units.values ():
            # add to the counts
            destroyed [unit.getOwner ()] += 1

        # get the ratio
        ratio = server.properties.module_unitsdestroyed_percantage

        # do we have more rebel units destroyed than the ratio?
        if destroyed[REBEL] > 0 and float (ok[REBEL]) / (destroyed[REBEL] + ok[REBEL]) < ratio:
            # rebel army destroyed
            rebeldestroyed = 1
        else:
            # not yet destroyed
            rebeldestroyed = 0

        # do we have more union units destroyed than the ratio?
        if destroyed[UNION] > 0 and float (ok[UNION]) / (destroyed[UNION] + ok[UNION]) < ratio:
            # union army destroyed
            uniondestroyed = 1
        else:
            # not yet destroyed
            uniondestroyed = 0

        # no action yet
        actiondata = []

        # are both destroyed?
        if rebeldestroyed and uniondestroyed:
            # both armies destroyed, a draw
            print "UnitsDestroyed.execute: rebel and union armies destroyed, draw"
            actiondata.append ( EndGameAct ( constants.BOTH_DESTROYED, step ) )

        elif rebeldestroyed:
            # rebel army destroyed
            print "UnitsDestroyed.execute: rebel army destroyed"
            actiondata.append ( EndGameAct ( constants.REBEL_DESTROYED, step ) )
            
        elif uniondestroyed:
            # union army destroyed
            print "UnitsDestroyed.execute: union army destroyed"
            actiondata.append ( EndGameAct ( constants.UNION_DESTROYED, step ) )
        
        # we have no action to send out
        return actiondata


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
