###############################################################################################
# $Id: engine.py,v 1.72 2002/09/07 19:08:33 chakie Exp $
###############################################################################################

import sys
import random
import copy
import types

import scenario
from math                          import log
from queue                         import Queue
from listhash                      import ListHash
from sequence                      import Sequence

# AI modules
from ai.assign_targets             import AssignTargets
from ai.clear_targets              import ClearTargets
from ai.timeout                    import Timeout
from ai.morale                     import Morale
from ai.rally                      import Rally
from ai.rest                       import Rest
from ai.schedule_combat            import ScheduleCombat
from ai.resolve_combat             import ResolveCombat
from ai.resolve_melee              import ResolveMelee
from ai.units_destroyed            import UnitsDestroyed

# needed action
from action.done_act               import DoneAct

# executor modules
import executor.factory
               
class Engine:
    """
    This class is the main logic of the entire application. It works as the game engine and
    processes all commands the players have given and sends out what actually should happen. It
    takes care of issues such as:

    * movement of units
    * all combat resolution
    * line of sight
    * status changes for units
    * a lot more

    It loops through all the plans that the players have issued to units, and calculates the
    action. 
    """

    def __init__ (self):
        """Initializes the instance. Sets default values for all needed members."""
 
        # plans and orders that are not assigned to any unit
        self.plans = []

        # combat that is registered for the current step
        self.combat = []
       
        # initialize the modules
        self.modules = [ AssignTargets (),
                         ClearTargets (),
                         #ScheduleCombat (),
                         ResolveCombat (),
                         ResolveMelee (),
                         Timeout (),
                         Rally (),
                         Rest (),
                         UnitsDestroyed (),
                         Morale () ]

        # no executors yet
        self.executors = {}

        # a list for generated action data for this turn
        self.action = []

        # start the plan ids from 0
        self.planid = 0
        
        print "Engine.init: engine initialized"


    def getAction (self):
        """Returns the list of action data that has been calculated for this turn. The list is reset
        when update() is called. This data should be viewed by the clients in the action phase."""
        return self.action
    

    def getCombatSchedule (self):
        """Returns the interna list of units that should fire this step. If something external to
        the engine (such as executors) need to have some combat performed a given step they just
        call this method and append a Unit instance to the end of the list. The units in this list
        will later have the combat executed by an AI module.

        The reason for having this list is that as all combat is performed by an AI module which
        externals can not directly access, there has to be some other means of registering
        combat. The AI module will get the attacking units from this list too.
        """
        return self.combat
    

    def update (self):
        """This method is the main workhorse of the entire engine. It checks all plans that should
        be processed this actual turn and processes them. This method should be called once after
        each turn has been finished.

        This method will make a deep copy of all the units.
        """

        print "Engine.update: starting calculations for turn %d" % scenario.time.getTurn ()

        # reset the game timer
        scenario.time.firstStep ()

        # no action for this turn yet
        self.action = []
                
        # create a deep copy of all units. this makes it possible for the engine to alter the units
        # while calculating action
        self.units        = copy.deepcopy ( scenario.info.units )
        self.selectedcopy = copy.deepcopy ( scenario.selected )

        # refreshes the units that the executors contain. the executors have a reference to a unit
        #in them, and that reference becomes dangling after the deep copies that are done here
        for executor in self.executors.values ():
            # refresh the unit(s) for this executor
            executor.refreshUnit ()
 
##         for unit in scenario.info.units.values ():
##             if unit.getPlans () != []:
##                 print "Engine.update: #### ", unit
##                 for p in unit.getPlans ():
##                     print "\t", p

        # loop while there are more steps in the current turn
        while 1:
            # get the current step
            step = scenario.time.getStep ()

            # no registered combat yet
            self.combat = []

            # calculate stuff
            self.updateForStep ( step )
            
            # execute all AI modules that are registered, and that need to be executed this step
            self.executeAI ( step )

            scenario.dispatcher.emit("engine_execute_iteration", {})
            
            # can the step be advanced?
            if not scenario.time.nextStep ():
                # we've reached the last step, so break out
                break

        # now we need to clear out dangling executors for units that have been destroyed. if we
        # don't do this then we will have a crash when we next time get here and try to refresh the
        # units above in this method

        # loop over all existing executors
        for planexec in self.executors.values ():
            # get the unit the executor has and see if it's destroyed
            if planexec.getUnit ().isDestroyed ():
                # oh, it has gone away, so this executor shouldn't be here either
                del self.executors[planexec.getUnit ().getId ()]

##         for unit in scenario.info.units.values ():
##             if unit.getPlans () != []:
##                 print "Engine.update: **** ", unit
##                 for p in unit.getPlans ():
##                     print "\t", p
                        
        # we're done, copy back the original units as they were before we started calculating the
        # action. we could also make sure that all parts of the engine work on the copy instead of
        # the original (scenario.info.units), and then just discard the copy, as that would save one
        # copy. There's however quite a lot of code that uses the global thing already, and I don't
        # feel like patching that.
        scenario.info.units    = copy.deepcopy ( self.units )
        scenario.selected = copy.deepcopy ( self.selectedcopy )


    def updateForStep (self, step):
        """Executes active plans for all units for the given 'step'. Loops through each unit and
        checks wether they have any plans that are currently being executed. If they have such a
        plan executor it is executed. Finished executors are removed.

        If the unit has no active executor the delay for the first plan is checked to see if it can
        be started. For a plan that can be executed a new Executor instance is created for that
        particular plan. A plan that can not yet be executed (delay not 0), the delay is just
        decremented. """
        
        # loop over all units and see wether the plans they have should be run
        for unit in scenario.info.units.values ():
            
            # do we have an active executor for this unit, i.e. a plan that is currently being
            # executed? if so it should be used
            if self.executors.has_key ( unit.getId () ):
                # yes, an active executor exists for this unit, run it
                self.runExecutor ( unit, step )
                
            else:
                # the unit has no current executor, so we need to check wether the unit has any
                # plans at all, and wether the one first in line can be started

                # get the unit active plan
                active = unit.getActivePlan ()
                
                # has it any plans?
                if active == None:
                    # no plans, next unit
                    continue

                # is the delay 0, i.e. is the plan ready for execution?
                if active.getDelay () == 0:
                    # yes, the plan has now suffered its delay, it can be executed. for this we need
                    # a new executor that knows how to execute the plan
                    #print "Engine.updateForStep: new active plan:", unit, active, unit.getPlans ()

                    # create a new plan executor for the active plan
                    planexec = executor.factory.create ( unit, active )

                    # make this executor the default for the unit
                    self.executors[ unit.getId () ] = planexec

                    # assign an id to the plan so that we can later identify it
                    active.setId ( self.planid )
                    self.planid += 1
                    
                    #print "Engine.updateForStep: executors:", self.executors
                   
                else:
                    # the plan was not yet ready to be executed, so decrement to delay
                    active.decrementDelay ()


    def runExecutor (self, unit, step):
        """Executes the currently active executor for the given unit. If the executor does not match
        the active plan (i.e. the first one), then the executor is supposed to have been cancelled
        or superceded somehow, and is deleted.

        An existing valid executor is executed and the optional action is stored locally for
        transmitting to the player later. If the plan becomes finished after execution, then the
        executor is removed.
        """
        # get the executor
        planexec = self.executors[ unit.getId () ]

        # get the ids of all the plans that the unit already has
        ids = map ( lambda p: p.getId(), unit.getPlans () )
        
        # has the executor been cancelled from the outside, ie. has a some module or similar
        # removed the unit's plans?
        if not planexec.getPlan ().getId () in ids:
            # the executor isn't among the plans for the unit, so it has been cancelled
            # somehow
            print "Engine.runExecutor: *** executor has been cancelled:", planexec.getPlan().getName()
            print "Engine.runExecutor: *** plan ids", ids
            print "Engine.runExecutor: *** plan id", planexec.getPlan ().getId ()
            del self.executors[ unit.getId () ]

            # we're done
            return


        # put a unique id for each plan instead?


        # let it execute the plan for this step if the planexec isn't already finished. it may have
        # been marked as finished during the refreshUnit() stage
        if not planexec.isFinished ():
            # execute it
            actiondata = planexec.execute ( step )

            # did we get any action?
            if actiondata:
                # did we get a single action or a list/tuple of actions? the methid may return
                # either a single action of a list/tuple of actions
                if type(actiondata) in (types.ListType, types.TupleType):
                    # a list of tuple, so add all elements from the list
                    self.action.extend ( actiondata )
                else:
                    # a single action, store the action for later sending to the clients
                    self.action.append ( actiondata )

        # is it finished? note that this can not be an "else", as the plan may have been finished
        # during execution
        if planexec.isFinished ():
            # yep, remove it from the dictionary of active executors. we're now supposed to
            # start working on the next plan for the unit. the new plan will be activated
            # next turn
            print "Engine.runExecutor: executor finished:", planexec.getPlan().getName()
            del self.executors[ unit.getId () ]

            # remove the plan from the unit too. this is the first plan, so some nice
            # slicing will work just fine
            unit.setPlans ( unit.getPlans() [1:] )

            #print "Engine.runExecutor: unit plans:", unit.getPlans ()
            
            # the plan is done, let the units know that too
            self.action.append ( DoneAct ( step, unit.getId () ) )
        

    def executeAI (self, step):
        """This method performs various simple automatic AI tasks for this step. It loops over all
        loaded modules and checks wether they want to be executed at this step. Not all modules are
        executed every step.

        Modules return a list of actions that they want to have sent to the clients. The action is
        added to the internal list of action.
        """

        # loop over all modules
        for module in self.modules:
            # is the module ready to run?
            if module.isReady ( step ):
                # yep, execute and get the optional action data
                actiondata = module.execute ( step )

                #print "Engine.executeAI: at %d executing ai module %s" % (step, module.getName ())

                # did we get any action data?
                if len ( actiondata ) > 0:
                    # yep, add it
                    self.action.extend ( actiondata )


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
