###############################################################################################
# $Id: change_mode_exec.py,v 1.10 2002/09/04 17:30:02 chakie Exp $
###############################################################################################

import scenario
import properties
from executor                    import Executor


def create (unit, plan):
    """This function is a factory for creating instances of the class 'Changemode'. Use this when you need
    to create instances. The new instance is returned."""
    # create a new instance and return it
    return ChangeModeExec (unit, plan)


class ChangeModeExec (Executor):
    """
    This class executes the plan 'changemode'. This plan is sent by clients when a unit has been
    ordered to change it mode. Modes can by the player be changed like this:

    * limbered -> unlimbering -> unlimbered
    * mounted -> dismounting -> dismounted
    * formation -> changing to formation -> column

    And of course the reverse too. This plan will alter the mode of the unit, i.e. basically toggle it.
    """

    def __init__ (self, unit, plan):
        """Initializes the instance."""
        # call superclass constructor
        Executor.__init__ ( self, unit, plan )

       
    def execute (self, step):
        """Executes the plan. Sets the intermediate mode for the unit, i.e. the unit to use while
        the mode is being changed. After the mode change is complete then the target mode is
        set. The intermediate mode is active for a certain period of time. If the unit already is in
        the wanted state then nothing will be done.

        The unit will gain fatigue while it is changing the mode, and that fatigue action is
        returned as long as the plan is executed.
        """
        
        # is this the first time we're executing the plan?
        if self.firsttime:
            # not first time anymore
            self.firsttime = 0

            # now we're setting the intermediate mode. we let the unit do that for a few steps, so
            # store the number of steps we will wait. the default is one minute
            self.wait = 60 / properties.seconds_per_step

            # create the action that sets the intermediate mode
            return self.setUnitMode ( self.unit, step, self.unit.getMode ().onChangeMode () )

        # not the first time we're executing, so are we done yet?
        if self.wait <= 0:
            # yep, the unit has changed mode now, set the final changed mode. this is the mode that
            # the unit gets after its intermediate mode, i.e. the one it was changing to in the
            # first place

            # execution is finished
            self.finished = 1

            # return the new mode changing action
            return self.setUnitMode ( self.unit, step, self.unit.getMode ().onDone () )

        # no, decrement the counter that indicates how long the unit is doing the 'reorganization' 
        self.wait -= 1

        # get the base fatigue change for the mode
        fatigue = self.unit.getMode ().getBaseFatigue ()

        # modify the value by the terrain too. it is more exhausting to change the mode when in
        # woods than when on an open field
        fatigue *= scenario.map.getTerrain ( self.unit.getPosition ()).movementFatigueModifier ( self.unit )

        # did we get any fatigue?
        if fatigue > 0:
            # yes, add the fatigue for the unit
            self.unit.getFatigue ().addValue ( int(fatigue) )

            # create action for that too and return it
            return self.changeModifiers ( self.unit, step )


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
