###############################################################################################
# $Id: executor.py,v 1.15 2002/09/07 19:08:34 chakie Exp $
###############################################################################################

import properties
import scenario

from mode.mode_factory           import createMode
from action.set_mode_act         import SetModeAct
from action.change_modifiers_act import ChangeModifiersAct

class Executor:
    """
    This class is a baseclass for all available types of plan executors. It is supposed to be
    subclassed for each plan that is to be executed. So if there is a plan 'Move' then there should
    be a 'MoveExec' that knows how to execute that plan. Each executor knows the unit that owns the
    executed plan and the plan itself.

    When the executor is created the plan is assumed to be ready to be executed, no extra delays
    needed. 

    The method 'execute()' is the method that executes the plan, and it is called once for each step
    that the plan is being executed. This methis should return an subclass of Action that contains
    the action for that single step, or None if no action is wanted.

    Each executor knows all units, and should alter the affected units before returning the action
    instance from 'execute()'. The reasoning behind this is that each executor when it executes does
    not need to worry where a unit is supposed to be at that step. Instead one can think that it has
    already applied all earlier computed action.
    """
    
    def __init__ (self, unit, plan):
        """Initializes the instance. Stores the unit and the plan this executor handles."""
        # store the passed data
        self.unit = unit
        self.plan = plan
        
        # not finished yet
        self.finished = 0

        # execution not yet started (for those who need this)
        self.firsttime = 1
        

    def isFinished (self):
        """Returns 1 if the execuor has determined that the plan has been finished. Returns 0 if it
        should still be executed."""
        return self.finished
    

    def getUnit (self):
        """Returns the unit that the executed plan belongs to."""
        return self.unit


    def refreshUnit (self):
        """Refreshes the unit the executor belongs to. This means that we refetch the unit from the
        global structures. The reason for this is that we do some deep copies, and these executors
        live over several turns. Thus the unit we have in the executor can be left 'dangling'."""
        # just fetch it again
        try:
            self.unit = scenario.info.units [ self.unit.getId () ]
            print "Executor.refreshUnit: refreshed:", self.unit
        except:
            # ok, the unit is probably dead
            print "Executor.refreshUnit: failed to refresh unit: %d. ok: %d" % ( self.unit.getId (),
                                                                                 self.unit.isDestroyed() )
            raise
        

    def getPlan (self):
        """Returns the plan that is being executed."""
        return self.plan


    def execute (self, step):
        """Executes the plan for this step. This method must be overridden by subclasses. The method
        may return either a single Action subclass, or a list/tuple of the same."""

        # must override!
        raise NotImplementedError ( "Executor.execute: this method must be overridden")


    def setUnitMode (self, unit, step, modename):
        """This is a convenience method for setting a new mode for a unit based on a mode name. The
        mode is set immediately for the unit that the executed plan refers to and the correct action
        is returned."""

        # create the proper mode instance
        newmode = createMode ( modename )

        # assign it to the unit
        unit.setMode ( newmode )

        # create the action that sets the new mode for clients too and return it
        return SetModeAct ( step, unit.getId (), modename )


    def changeModifiers (self, unit, step):
        """This method is a convenience method for subclasses that need to generate action for
        changed modifiers. The call to make the action is a bit long and unwieldy, so it has been
        put here instead. Returns a new ChangeModifiersAct."""

        # create the action
        return ChangeModifiersAct ( unitid=unit.getId (), step=step,
                                    fatigue=unit.getFatigue ().getValue (),
                                    morale=unit.getMorale ().getValue (),
                                    experience=unit.getExperience ().getValue () )

    
    def validateUnit (self, id):
        """Validates that a given unit is valid. Tries to get the unit with the given id from the
        unit collection. Returns the unit if found and None if not found."""
        # do we have such a unit?
        if not scenario.info.units.has_key ( id ):
            # no such unit
            return None
        
        # get it!
        return scenario.info.units [ id ]
    

#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
