###############################################################################################
# $Id: skirmish_exec.py,v 1.12 2002/09/04 16:44:34 msa Exp $
###############################################################################################

import scenario
import properties
import rotate_util       
import move_util       

from executor                    import Executor
from action.rotate_act           import RotateAct
from action.set_target_act       import SetTargetAct


def create (unit, plan):
    """This function is a factory for creating instances of the class 'SkirmishExec'. Use this when
    you need to create instances. The new instance is returned."""
    # create a new instance
    return SkirmishExec ( unit, plan ) 


class SkirmishExec (Executor):
    """
    This class implements an executor for the plan 'Skirmish'. It contains the actual code for
    skirmishing with a unit. The assaulting is performed in a few steps:

    1. set an skirmish mode
    2. make the unit face its target
    3. if the unit is too far from the target to fire it will finish the plan and idle
    4. if the unit is within range it will start the skirmish

    Note that this executor will not perform the actual skirkishing, it will only face the unit in
    the correct direction, set a target and put the unit in a skirmishing mode. An AI module will
    then take care of the actual firing.
    """

    def __init__ (self, unit, plan):
        """Initializes the instance."""
        # call superclass constructor
        Executor.__init__ ( self, unit, plan )
       
        # get the target we're skirmishing
        self.target = scenario.info.units [plan.getTargetId ()]
       
        # calculate our turning speed
        self.turnspeed = rotate_util.calculateTurnSpeed ( self.unit )

        # store the current x and y values, these get incremented by the deltas
        self.currentx, self.currenty = self.unit.getPosition ()
        
        # get the target position, i.e. where we're heading. if we're not close enough to the target
        # we must move towards it. the unit will never move all the way to the target, instead it
        # will just move closer, and when close enough, do the final assault
        self.targetx, self.targety = self.target.getPosition ()
        
        # calculate the angle the unit should rotate to
        self.targetfacing = rotate_util.calculateAngle ( self.currentx, self.currenty,
                                                         self.targetx, self.targety )

        # get the current facing for the unit
        self.currentfacing = self.unit.getFacing ()
        
        # set the initial delay
        self.setFiringDelay ()

  
    def refreshUnit (self):
        """Refreshes the unit the executor belongs to. This means that we refetch the unit from the
        global structures. The reason for this is that we do some deep copies, and these executors
        live over several turns. Thus the unit we have in the executor can be left 'dangling'. This
        method also refreshes the target."""

        # to the basic refresh first
        Executor.refreshUnit ( self )
        
        try:
            self.target = scenario.info.units [ self.target.getId () ]
        except:
            # ok, the unit is probably dead
            print "Executor.refreshUnit: target failed: %d. ok: %d" % ( self.target.getId (),
                                                                        self.target.isDestroyed() )
            raise
       
         
    def execute (self, step):
        """Executes the plan for this step. There are a few things that need to be done, in this
        order:

        o set an skirmish mode
        o make the unit face its target
        o if the unit is too far from the target it will cancel the skirmishing
        o if the unit is within range it will start the skirmish by setting a target and then
          ending the plan. 

        Actual firing is done by the combat modules in the engine. This plan merely sets a nice
        skirmish mode and assigns a target to the unit.
        """
        
        # is this the first time we call this method?
        if self.firsttime:
            # this is the first time called, setup for execution
            self.firsttime = 0

            # set unit in skirmish mode 
            return self.setUnitMode ( self.unit, step, self.unit.getMode ().onSkirmish () )

        # is the target still alive, within range and visible? we may have lost it during the
        # turning phase, i.e. the unit has moved away, been destroyed or something similar
        if not self.unit.seesEnemy ( self.target ) or not self.unit.inRange ( self.target ) or \
           self.target.isDestroyed ():

            # oops, can't skirmish with that one anymore
            print "SkirmishExec.execute: **** target %d lost for %d" % (self.target.getId (),
                                                                        self.unit.getId () )
            # this one is done
            self.finished = 1
            
            # set unit in whatever mode it'll have after the skirmish is done
            return self.setUnitMode ( self.unit, step, self.unit.getMode ().onDone () )


        # does the unit have the correct target set?
        if self.unit.getTarget () != self.target:
            # the unit does not have the same target as us, set it
            self.unit.setTarget ( self.target )

            # and return the same action too
            return SetTargetAct ( step, self.unit.getId (), self.target.getId () )


        # is the rotation done?
        if self.currentfacing != self.targetfacing:
            # no, first get the new facing for the unit
            self.currentfacing = rotate_util.rotate ( self.currentfacing, self.targetfacing, self.turnspeed )

            # assign the new facing to the unit too
            self.unit.setFacing ( int (self.currentfacing) )

            #  create action for the changed facing and return it. we're done for this step
            return RotateAct ( step, self.unit.getId (), int (self.currentfacing) )


        # we got this far, now the unit can start firing as fast as it can. decrement the delay and
        # see if we reach 0
        self.delay -= 1

        # did we reach 0 now?
        if self.delay == 0:
            # we're ready to fire, set new delay
            self.setFiringDelay ()

            # we want action, make sure the engine takes care of that
            scenario.engine.getCombatSchedule ().append ( (self.unit, self.target) )


    def setFiringDelay (self):
        """Assigns the firing delay for the unit. The delay is used when checking how often the unit
        can fire during one turn. The delay has a base delay and then up to 0-10 steps added
        depending on the unit experience."""

        # do the funky math
        self.delay = 10 + int ( (100.0 - self.unit.getExperience ().getValue ()) * 0.01)

 
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
