###############################################################################################
# $Id: event_loop.py,v 1.99 2002/09/05 16:58:48 msa Exp $
###############################################################################################

import sys
import time
import string
import copy
import pygame
from pygame.locals import *

import scenario
import constants
import properties
import net.connection
import net.connection_poller_thread

import state.idle
import state.error
from gui.end_game import EndGame

import plan.factory
import action.factory
import immediate.factory


#################################################################################################
def endGame ():
    """This function is called when the main loop has ended and the end screen should be shown. This
    function will take care of creating and handling the end dialog and eventually quit the
    application. Also terminates the music. If the ending was a crash then this will just exit the
    game.""" 

    # how did the game end?
    if scenario.end_game_type == constants.CRASH:
        # we crashed, abandon ship, abandon ship
        sys.exit ( 1 )
        
    # create and run the dialog
    EndGame ().run ()

    # stop the music
    scenario.audio.stopMusic ()

    
#################################################################################################
def crash ():
    """This function is called when we have a server crash. It prints some info and exits."""
    print "crash: server seems to have crashed, exiting"
    print "crash: this should save state or do something intelligent"
    sys.exit ( 1 )
    

#################################################################################################
def updateDisplay ():
    """Updates the main display if there are any suitable events in the event queue. If there are
    they are applied and then the display is repainted to reflect the new changes. This method will
    periodically take care of animating stuff on the screen. This is also the sole point of the main
    running application where the actual screen is visible updated. This is good as we then don't
    have updates all around the sources, leading to duplicate updates."""

    # repaint the playfield if needed. This is the only location in the entire event loop
    # the playfield is really repainted, all other places merely register interest int having
    # something painted
    scenario.playfield.paint ()

    # now update the display to reflect all changes
    scenario.sdl.update ()


#################################################################################################
def handleImmediateUpdate (update):
    """Handles an immediate update that has arrived on the network. The update is sent to the
    currenttly active game state for handling."""

    # execute it in case it has something meaningful
    update.execute ()

    # let the current state handle it
    newstate = scenario.current_state.handleImmediate ( update )

    # do we have a new state that should be active
    if newstate:
        # yep, set it
        scenario.current_state = newstate

 
#################################################################################################
def handleAction (act):
    """Handles an action that has arrived on the network. The action is stored in the global data
    structure for updates for later execution."""

    # do we have some earlier action for the particular step of the action?
    if not scenario.events.has_key ( act.getStep () ):
        # nope, so create an empty list there
        scenario.events [ act.getStep () ] = []

    # now append it
    scenario.events [ act.getStep () ].append ( act )



#################################################################################################
def pumpPygame ( params ):
    """This is used as a callback hook in various places, so it will pump the pygame event queue
    and update the display."""
    pygame.event.pump()


   
#################################################################################################
def checkNetEvents ():
    """Checks wether there are any network events ready to be received. If there is something to be
    handled it is done. Handled means that a line of data is read from the connection and a new
    command is. If a critical error is encountered (i.e. can't read from socket) then crash() is
    called."""

    # get a line of data if one is availabel
    try:
        line = scenario.connection.readLine ()
    except IOError:
        # failed to read from the socket, remote host closed down?
        scenario.current_state = state.error.Error ()
        return 0
        #crash ()
    
    # did we get anything?
    if line:
        #print "checkNetEvents: got '%s'" % line

        # split the data
        parts = line.split ()

        # did we get any data at all?
        if len ( parts ) == 0:
            # oops, nothing there?
            raise "checkNetEvents: no parameters at all?"
       
        # have the plan factory try to make an immediate update
        update = immediate.factory.create ( parts )

        # did we get an update?
        if update != None:
            # yes, it was an update, handle it
            handleImmediateUpdate ( update )
            return 1

         
        # have the action factory try to make an action instance
        act = action.factory.create ( parts )

        # did we get an update?
        if act != None:
            # yes, it was an update, handle it
            handleAction ( act )
            return 1

       
        # no immediate update and no action, have the plan factory make a plan
        newplan = plan.factory.create ( parts )

        # this new plan should be put somewhere smart. the normal thing here is that the plans are
        # sent be the remote cliet (human/ai) after the turn has been ended. the plan should
        # probably be put to the unit it concerns?

        # when are the plans for the units removed/cleared? maybe through an action that is
        # calculated by the engine?

        # add the plan to the unit it belongs to
        scenario.info.units [ newplan.getUnitId () ].getPlans ().append ( newplan )
           
        #print "checkNetEvents: got '%s'" % newplan.toString ().strip ()
        return 1
    
    return 0

        
#################################################################################################
def nextEvent():
    """Waits and returns the next event from pygame. If it is a mouse motion event, take all those
    events and only use the last one.  This means the engine doesn't get every single mouse motion
    event, and makes refreshing smoother when moving the mouse quickly.  """

    event = pygame.event.wait ()
    if event.type == MOUSEMOTION:
        mousemoves = pygame.event.get (MOUSEMOTION)
        if mousemoves != []:
            event = mousemoves [-1]
    return event


   
#################################################################################################
def event_loop ():
    """Main function that handles the entire event loop. Polls events from the SDL event system and
    handles them. Regularly checks wether there is any incoming data to be read."""
    
    # set the default state
    scenario.current_state = state.idle.Idle ()
    newstate = None
    
    # prepaint the playfield
    scenario.sdl.fill ( (0,0,0) )
    scenario.playfield.paint ()

    # update it all
    scenario.sdl.update ()

    # get current turn
    turn = scenario.time.getTurn ()

    # Create the thread that listens for incoming client/server data
    connthread = net.connection_poller_thread.ConnectionPollerThread(scenario.connection)
    connthread.start()

    # perform backup of the unit's states before we start giving orders. giving orders changes them, and we
    # need a pristine copy of the units
    for unit in scenario.info.units.values ():
        # push the state internally
        unit.pushState ()

    read_net = 0

    # Pump pygame event queue when we otherwise wouldn't be in the main loop
    scenario.dispatcher.registerCallback("engine_execute_iteration", pumpPygame )
    scenario.dispatcher.registerCallback("send_action_data", pumpPygame )
    
    # loop forever
    while scenario.playing != constants.GAME_ENDED:
        # This is quite a bad hack.
        # The problem is that we send one USEREVENT from
        # net.ConnectionPollerThread, even though
        # there might be several events arriving
        while read_net:
            #print "Check net events"
            if not checkNetEvents ():
                read_net = 0

        # get next event
        event = pygame.event.wait ()

        # Currently, NOEVENT is used for the State.callMeSoon() logic
            
        if event.type == QUIT:
            # user wants to quit the application, so do it by returning from the main event loop
            scenario.playing = constants.GAME_ENDED
            return

        elif event.type == USEREVENT:
            # do we have any network events ready?
            # TODO: should this be called more often? No we potentially repaint for every net event we
            # get, and that is bad. Indent one more step?
            
            # Perhaps timestamp messages and send them when a) a buffer is full, or b) a
            # certain timeout has been exceeded ? Then the receiver can take several
            # messages between two refreshes
            if event.code == properties.USEREVENT_DATA_FROM_SOCKET:
                # Read network events later, there might
                # be several!
                read_net = 1
            else:
                print "BUG - Undefined USEREVENT"
                    
        elif event.type == USEREVENT + 1:
            # a timer event, let the dispatcher handle all optional timer callbacks
            scenario.dispatcher.checkTimers ()
 
        # workaround to handle mouse movements. this is true if the mouse has been moved AND the
        # state wants the movement events OR it's another event type
        elif (event.type == MOUSEMOTION and scenario.current_state.wantMouseMotion ()) or \
             event.type != MOUSEMOTION:
                        
            # handle the event and get the possible changed state
            newstate = scenario.current_state.handleEvent ( event )

            # do we have a new state that should be active?
            if newstate:
                # yep, set it
                scenario.current_state = newstate

        # update the display
        updateDisplay ()


    # game ended, show what needs to be shown
    endGame ()

     
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
