###############################################################################################
# $Id: gametime.py,v 1.24 2002/05/06 12:45:51 chakie Exp $
###############################################################################################

import sys
import time
import scenario
import pygame
import properties

class GameTime:
    """

    This class defines the current time in the game, such as the current turn, number of turns for
    one game minute, the starting time of the battle, as well as the current game time. Each time
    the timer of the game ticks one second forward this class should be updated.

    One game minute is divided into a number of turns. One turn is always one second, and is the
    smallest time unit the game handles.

    Apart from the 'fake' time this class also maintains the current realtime second of the
    game. This time is the current second of the real system clock. Use the method
    hasRealtimeAdvanced() to check wether we've entered a new second or not.
    
    This class does not do any kind of repainting.
    """

    def __init__ (self, hour = 0, minute = 0, second = 0, turn = 0):
        """Creates the instance of the class. Sets default values for all members. If the parameter
        'turn' is set then the timer will start up with a default turn."""

        # set passed/default hour, minute and second
        self.hour = hour
        self.minute = minute
        self.second = second

        # store the total minutes since 00:00:00
        self.totaltime = hour * 60 + minute

        # current turn
        self.turn = turn

        self.last_animation_update = -1
 
        # no animation step yet performed
        self.step = 0

        # the maximum number of steps per turn
        self.maxstep = ( properties.minutes_per_turn * 60 ) / properties.seconds_per_step - 1
        

    def getTime (self):
        """Returns the current time of the game. This is a tuple (hour,minute) and is provided for
        convenience. """
        return ( self.getHour(), self.getMinute(), self.getSecond() )

    
    def getHour (self):
        """Returns the current hour of the time."""
        return self.hour + self.getMinute() / 60


    def getMinute (self):
        """Returns the current minute of the time."""
        return self.minute + ( properties.seconds_per_step * self.step ) / 60


    def getSecond (self):
        """Returns the current second of the time."""
        return ( properties.seconds_per_step * self.step ) % 60


    def getTicks (self):
        """Returns the pygame ticks in milliseconds."""
        return pygame.time.get_ticks ()

 
    def getTurn (self):
        """Returns the current turn."""
        return self.turn


    def advanceTurn (self):
        """Advances the current turn one turn. Increments the current turn and adds the number of
        minutes that a turn takes to the count of turn. No checks are done on the number of turns
        etc, that is the responibility of some other party. Returns the new turn"""
        
        # add to turns
        self.turn += 1

        # reset the step
        self.step = 0
        
        # yep, the minutes too
        self.addMinutes ( properties.minutes_per_turn )

        # return new turn
        return self.turn


    def getStep (self):
        """Returns the current step."""
        return self.step


    def setStep (self, step):
        """Sets the current step."""
        # precautions
        if step < 0 or step > self.maxstep:
            # invalid step
            raise "GameTime.setStep: the step value %d is invalid" % step
        
        self.step = step


    def nextStep (self):
        """Advances the current action step one forward if it is possible. If the step has already
        reached the last possible step for this action phase then nothing is done and 0 returned."""
        # can we advance the step?
        if self.step == self.maxstep:
            # no can do
            return 0

        # all ok, advance it
        self.step += 1

        # emit a signla telling the world that we have a new step
        if not scenario.local_player_ai:
            scenario.dispatcher.emit ( 'new_step', self.step )
       
        return 1


    def prevStep (self):
        """Moves the current action step one backward if it is possible. If the step has already
        reached 0 for this action phase then nothing is done and 0 returned."""
        # can we advance the step?
        if self.step == 0:
            # no can do
            return 0

        # all ok, advance it
        self.step -= 1
        return 1


    def lastStep (self):
        """Moves the current action step to the last available."""
        self.step = self.maxstep


    def firstStep (self):
        """Moves the current action step to the first available, i.e. 0."""
        self.step = 0
        

    def getMaxStep (self):
        """Returns the max step without altering the current step. There is no getMinStep(), as the
        minimum is always 0."""
        return self.maxstep


    def addMinutes (self, minute):
        """Adds 'minute' number of minutes to the current minutes. Wraps the minutes to hours if
        needed."""
        # add new minute
        self.minute += minute

        # did we wrap? do a loop in case we added several hours worth of minutes
        while self.minute > 59:
            # reduce one hour from the minutes and add one hour to the hours
            self.minute -= 60
            self.hour += 1
            
        # set new totaltime
        self.totaltime = self.hour * 60 + self.minute


##     def needAnimationUpdate (self):
##         """Returns 1 if the current time has advanced so much that animation should be updated and 0
##         if not. The time between updates is properties.animation_interval. If an update is needed
##         then the current time is stored internally, so two sequent calls to this method may not
##         return the same result."""
##         # get the current millisecond
##         millisecond = pygame.time.get_ticks ()

##         # do we need to make a initial update?
##         if self.last_animation_update == -1:
##             # yep, store this time as the last update time
##             self.last_animation_update = millisecond

##             return 1

##         # now, has more time elapsed then allowed?
##         if self.last_animation_update + properties.animation_interval < millisecond:
##             #  yep, store this time as the last update time
##             self.last_animation_update = millisecond

##             return 1

##         # no need to update yet
##         return 0
    


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
