###############################################################################################
# $Id: setup_network.py,v 1.24 2002/09/09 09:28:04 chakie Exp $
###############################################################################################

import pygame
import os
import getpass
import traceback
from socket         import *

import scenario
import properties

from net.connection import Connection
import gui.widget
from gui.image          import Image
from gui.dialog         import *
from gui.button         import Button
from gui.checkbox       import CheckBox
from gui.label          import Label
from gui.editfield      import EditField
from gui.widget_manager import WidgetManager
from gui.messagebox     import Messagebox
from gui.wait_client    import WaitClient

# return codes used to indicate what should happen
CANCEL = 0
OK     = 1

class SetupNetwork(Dialog):
    """
    This class is used as a dialog for setting up the connection to the game server. It asks for the
    user name (whatever name) and the name and port of the host where the server runs. The network
    connection is then initialized if all is ok.

    This dialog can optionally launch an AI client if run as a server. This means that the dialog
    intializes the game as a server, then immediately launches the AI which then connects to this
    instance. Makes it faster to start.
    """

    def __init__ (self, launch_ai=0):
        "Creates the dialog."
        # load the fonts too
        self.titlefont = pygame.font.Font  ( properties.titlefont, 24 )
        self.smallfont = pygame.font.Font  ( properties.textfont, 14 )

        # init superclass
        Dialog.__init__ (self, scenario.sdl)

        # set our background to a tiled image
        self.setBackground ( properties.window_background )

        # store the ai flag
        self.launch_ai = launch_ai

        

    def createWidgets (self):
        """Creates all widgets for the dialog. Depending on wether we're a server or a client
        different widgets will be shown."""

        # are we a server?
        if not scenario.run_as_server:
            # we're running as client. need an additional label and editfield for the hostname
            self.wm.register ( Label (self.smallfont, "Server host: ",   (250, 240), color=(255, 255, 255)) )

            self.host = EditField ( self.smallfont, "localhost", 200, (450, 240), color=(255,255,0),
                                    background = (80,80,80))

            self.wm.register ( self.host )

        # common widgets
        self.wm.register ( Label (self.titlefont, "Setup network information", (20, 10), color=(255, 0, 0)))

        self.wm.register ( Label (self.smallfont, "Server port: ",   (250, 300), color=(255, 255, 255)) )
        self.port = EditField ( self.smallfont, str(properties.network_port), 200, (450, 300),
                                color=(255,255,0), background = (80,80,80))

        self.wm.register ( self.port )
           
        # buttons
        self.wm.register ( Button ( properties.path_dialogs + "butt-ok-moff.png",
                                    properties.path_dialogs + "butt-ok-mover.png",
                                    (284, 650 ), {widget.MOUSEBUTTONUP : self.ok } ) )
        self.wm.register ( Button ( properties.path_dialogs + "butt-back-moff.png",
                                    properties.path_dialogs + "butt-back-mover.png",
                                    (528, 650 ), {widget.MOUSEBUTTONUP : self.back } ) )

        # if we're supposed to start immediately then enable the timer
        if scenario.commandline_quickstart != 0:
            self.enableTimer(500)
            self.quickstart_state = 0


    def timer(self):
        """Callback triggered when the timer has fired. This is used if Civil is supposed to be
        started immediately without any user intervention. This will simulate a mouse click in the
        XXX button when YYY."""
        if 0:
            pass

        elif self.quickstart_state == 0:
            pygame.event.post(pygame.event.Event(widget.MOUSEBUTTONDOWN, { "pos": (290, 670) }))
            
        elif self.quickstart_state == 1:
            pygame.event.post(pygame.event.Event(widget.MOUSEBUTTONUP, { "pos": (290, 670) }))

        else:
            self.disableTimer()

        # increment statemachine
        self.quickstart_state += 1
            
    
    def ok (self, trigger, event):
        """Callback triggered when the user clicks the 'Ok' button. Applies the changes after
        verifying the given data, and closes the dialog. Tries to connect to the remote server or
        waits for a remote client to connect to us, and if all is ok the dialog is accepted. If
        something fails the player may choose again. If some data is missing the player must choose
        again"""

        # get the port. this is common for both client and server
        try:
            port = int ( self.port.getText () )

            # is it valid?
            if not 1 <= port <= 65535:
                # not a valid number, show a messagebox
                Messagebox ( "Invalid port number, must be in the range 1 to 65535!" )

                # repaint and go away
                self.wm.paint (force=1, clear=1)
                return widget.HANDLED
            
        except TypeError:
            # not a valid number, show a messagebox
            Messagebox ( "Invalid port number!" )

            # repaint and go away
            self.wm.paint (force=1, clear=1)
            return widget.HANDLED


        # are we a server?
        if scenario.run_as_server:
            # init serverside socket
            return self.initServerSocket ( port )

        else:
            # init clientside socket
            return self.initClientSocket ( port )


    def initServerSocket (self, port):
        """Initializes a socket for listening on tghe given port, ie. server mode. If the AI client
        should be launched automatically then it is started immediately. While waiting for a client
        (or the AI) to connect a separate dialog is executed."""
        try:
            # create the socket
            s = socket (AF_INET, SOCK_STREAM)
            
            # allow bind()ing while a previous socket is still in TIME_WAIT.
            s.setsockopt ( SOL_SOCKET, SO_REUSEADDR, 1 )
            
            # bind our socket for incoming requests
            s.bind ( ( '', port ) )
        except:
            # not a valid number, show a messagebox
            Messagebox ( "Could not use socket! It may already be in use." )

            # repaint and go away
            self.wm.paint (force=1, clear=1)
            return widget.HANDLED

        # listen for clients
        s.listen ( 1 )

        # do we want an AI client?
        if self.launch_ai:
            # yes, do it
            self.__startAI ( port )

        # run a dialog that shows some nice info while we wait for the client to connect
        dialog = WaitClient ( s )
        dialog.run ()

        # set our status to the status of the executed dialog
        self.state = dialog.state

        # was the dialog rejected?
        if self.state == REJECTED:
            # yeah, close our socket then
            s.close ()
        
        # we're done anyway
        return widget.DONE


    def initClientSocket (self, port):
        """Connects to the server. If all is ok a connection is stored in the scenario. On error a
        dialog is shown to the user and the user may try again."""
        
        # get the name of the host
        host = self.host.getText ()

        # did we get a hostname?
        if host == "":
            # oops, no host given, show a messagebox
            Messagebox ( "No server hostname given!" )

            # repaint and go away
            self.wm.paint (force=1, clear=1)
            return widget.HANDLED

        try:
            # create the socket
            newSocket = socket (AF_INET, SOCK_STREAM)
        
            # connect to the remote system
            newSocket.connect ( ( host, port  ) )

            # all ok, store the new and connected socket and the extra info
            scenario.connection = Connection ( socket=newSocket, host=host, port=port )

        except:
            Messagebox ( "Could not connect to server!" )

            # repaint and go away
            self.wm.paint (force=1, clear=1)

            return widget.HANDLED
       
        # all is ok, we're accepting the dialog
        self.state = ACCEPTED
        
        return widget.DONE

               
    def back (self, trigger, event):
        """Callback triggered when the user clicks the 'Back' button. Simply closes the dialog and
        returns to the main dialog, ignoring any changes. This is used when the player wants to
        change something."""
        # we're cancelling the dialog
        self.state = REJECTED
        
        return widget.DONE


    def __startAI (self, port):
        """Launches the AI client, which will connect to this instance. The 'port' parameter is the
        port that the AI should use when connecting. """

        # BUG
        ### Duh, let the operating system go through $PATH and execute civil-ai
        ### Perhaps developers should add /home/to/the/civil/project first into their $PATH?

        # get the full path to the ai app
        command = properties.path_ai_client

        # linux, unix etc. assume that & works here.
        # if it won't work for some platform add new
        supported = {"linux":  self.__startAIUnix,   # tested
                     "bsd":    self.__startAIUnix,   # not tested
                     "darwin": self.__startAIOSX,    # osx
                     "win":    self.__startAIWindows # not tested
                     }

        # get the method to run
        method = supported [ properties.platform_name ]

        # and run it
        method (command, port)


    def __startAIUnix (self, command, port):
        """Performs the AI starting on Linux/Unix."""
        # add a & to make it go in the background
        #command = "./civil-ai.py --port=%d &" % port
        command += " --port=%d &" % port
        #command = "civil-ai --port=%d &" % port

        # and launch
        os.system ( command )


    def __startAIOSX (self, command, port):
        """Performs the AI starting on OSX """

        # Add the extension to the passed command...
        # command += '.py'
        
        # create the port parameter
        port = " --port=%d &" % port
        
        # and launch
        os.system ( '/usr/local/bin/python ' + command + port )

 
    def __startAIWindows (self, command, port):
        """Performs the AI starting on Windows."""

        # add the port data
        #command += " --port=%d" % port

        command = command.replace ('\\', '/')

        # execute using the windows specific function
        os.startfile ( command )
    

        
        
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
