###############################################################################################
# $Id: hex.py,v 1.54 2002/07/25 16:03:29 msa Exp $
###############################################################################################

import terrain
import properties
from hex_template import HexTemplate

class Hex:
    """
    This class defines a single hex in the map. A hex is the smallest possible location a unit can
    be positioned on. Each hex contains an icon id, the absolute heights of each of the six
    triangles in the hex and a reference to a template hex. The templates are shared among all
    hexes, and thus are present only once. They are used to query static data about a hex, such as
    terrain, color data, road/river info etc. As this data does not change there is no need to
    duplicate it unnecessarily

    Also provides functionality for returning a RGB color representing the hex.

    If a new hex is added the file 'terrains.txt' *must* be changed too.
    """

    # a dict of templates for all the terrain hexes
    templates = None

    
    def __init__ (self, iconid):
        "Initializes the hex and stores passed data."

        # do we already have the hex data loaded?
        if Hex.templates == None:
            # not yet loaded, so go do it
            self.loadHexTemplates ()

        # store the template that this hex uses as base
        self.template = Hex.templates[iconid]
 
        # set default height
        self.height = 0


    def loadHexTemplates (self):
        """Loads the static terrain data from a file and stores it all in an internal shared
        dictionary. Each hex will have a line that describes the terrain, height and roads/rivers
        for each triangle of the hex."""

        print "loading hex templates"
        
        # init the dictionary
        Hex.templates = {}
        
        count = 1
        
        # open the file for reading and read all lines
        for line in open ( properties.hex_data_file, "r" ).readlines ():
            # clean it up
            line = line.strip ()
            
            # is this a comment or an empty line?
            if len (line) == 0 or line[0] == "#":
                # and empty line, next please
                continue
            index = line.find("#")
            if index != -1:
                line = line[:index]

            # split it up into parts based on spaces
            parts = line.split ()

            # we should have three parts
            if len ( parts ) != 5:
                # illegal data
                raise "terrain data file %s is illegal, line %d (%s)" % (properties.hex_data_file,
                                                                         count, line)
            # get the id and split up the terrain types, road/river into and height
            id       = int ( parts[0] )
            terrains = parts[1]
            roads    = parts[2]
            heights  = parts[3]
            color    = parts[4].split (',')

            # create a new template
            template = HexTemplate ( id, terrains, roads, heights, color )

            # cache it in the shared dict
            Hex.templates [ id ] = template

            count += 1


    def getTerrains (self):
        """Returns the terrains for each triangle in the hex. The terrains are returned as a list of
        instances of Terrain."""
        return self.template.terrains


    def getTerrain (self, triangle):
        """Returns the terrain for th given triangle in the hex."""
        return self.template.terrains[triangle]

    
    def getColor (self):
        """Returns a tuple (red,green,blue) that represents the general color of the icon. This color
        can be used in for instance the minimap. If the icon does not have a color allocated then it
        returns a black color and prints a warning."""
        return self.template.color


    def getHeight (self):
        """Returns the absolute height of the hex."""
        return self.height


    def setHeight (self, height):
        """Set a new height for the hex."""
        self.height = height


    def getRoads (self):
        """Returns the roads of the triangles in the hex. If the hex has roads then the corresponding
        values in the returned list are 1, otherwise 0."""
        return self.template.roads


    def getRivers (self):
        """Returns the rivers of the triangles in the hex. If the hex has roads then the corresponding
        values in the returned list are 1, otherwise 0."""
        return self.template.rivers


    def getIconId (self):
        """Returns the id of the icon for this hex."""
        return self.template.id


    def toXML (self, x, y):
        """Returns a string containing the hex serialized as XML. The lines are not indented in
        any way, and contain no newline. As the hex does no longer know its position in the map (the
        hexes may be shared) the position is passed as parameters."""

        # build up the xml and return it
        return '<hex x="%d" y="%d" terrain="%d"/>' % ( x, y, self.template.id )

    
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
