###############################################################################################
# $Id: hex_template.py,v 1.7 2002/07/25 16:03:29 msa Exp $
###############################################################################################

import terrain
import properties


class HexTemplate:
    """
    This class defines a template for a given hex. It contains only static data that actual Hex
    class instances can refer to. This means that a lot of memory is saved by only storing the
    static non-changing data for a hex in one place.

    The data that this class contains is:

    * a set of Terrain instances, one for each triangle
    * a height for each triangle
    * a color that that be used to represent the hex

    The height is not absolute, it is instead a relative height among the triangles in the
    hex, which 0 being baseline height, negative values sunk down and positive values hills etc.
    """
    
    def __init__ (self, id, terrains, roads, heights, color):
        "Initializes the hex template and parses passed data."
        # store the id
        self.id = id

        # no terrains, heights now rivers/roads yet
        self.terrains = []
        self.heights  = []
        self.roads    = [ 0, 0, 0, 0, 0, 0 ]
        self.rivers   = [ 0, 0, 0, 0, 0, 0 ]

        # set color 
        self.color    = ( int (color[0]), int (color[1]), int (color[2]) )
        
        # parse and store all the data
        self.__setTerrains ( terrains )
        self.__setRoadsAndRivers ( roads )
        self.__setHeights ( heights )


    def getId (self):
        """Returns the id of the icon."""
        return self.id
    

    def getTerrains (self):
        """Returns a list of the terrain data for all the triangles. There is one instance of
        Terrain for each triangle."""
        return self.terrains


    def getTerrain (self, triangle):
        """Returns the terrain data for a given triangle. Valid values are 0..5."""
        return self.terrains[triangle]

    
    def getColor (self):
        """Returns a tuple (red,green,blue) that represents the general color of the icon. This color
        can be used in for instance the minimap. """
        return self.color
    

    def getDeltaHeights (self):
        """Returns a list of all the relative heights of the triangles in the hex compared to a
        baseline of 0."""
        return self.heights

    def getDeltaHeight(self, triangle):
        """Returns the height of a given triangle, 1-6, relative
        to the middle of the hex."""
        return self.heights[triangle-1]
            
    def __setTerrains (self, terrains):
        """Creates a list of Terrain instances based on the 'terrains' list. The parameter list is
        just characters that are used to describe a hex."""

        # loop over all the parts
        for code in terrains:
            # get the terrain instance and add it
             self.terrains.append ( terrain.terrainDict [code] )

    def getRoads(self):
        return self.roads

    def getRivers(self):
        return self.rivers

    def __setRoadsAndRivers (self, roads):
        """Parses the list of road and river data and stores road and river information
        locally. Initially we have no roads and no rivers. A letter 'p' in the list means that the
        triangle has a road in it, 'r' means a river and '-' means the triangle has nothing special."""
        index = 0
        
        # loop over all the parts
        for code in roads:
            # a road?
            if code == 'p':
                # yep, set a flag indicating a road
                self.roads [index] = 1

            elif code == 'r':
                # yep, set a flag indicating a river
                self.rivers[index] = 1

            index += 1
            

    def __setHeights (self, heights):
        """Stores the relative heights of each triangle. Each height in the list os given as a
        character in the range 0..9. The height are all offset by 5 so that we have positive values,
        meaning that 5 has to be subtracted befoere the value is used."""
        # loop over all the 
        for height in heights:
            # convert the character to an int and subtract the offset we used
            self.heights.append ( int (height) - 5 )


    
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
