###############################################################################################
# $Id: map.py,v 1.51 2002/09/07 11:39:16 msa Exp $
###############################################################################################

import re
import os
import pygame
from pygame.locals import *
from math    import floor, ceil, pi, atan2, sqrt

import properties
import scenario
from terrain import Terrain
import util.geometry

class MapHeightException(Exception):
    """Simple class for keeping x,y pairs of map height errors."""
    def __init__(self, pairs):
        self.value = pairs
        Exception.__init__(self)


class Map:
    """
    This class defines the basic map of the game. It contains a 2D matrix of the hexes in the game. 

    It also a list of 6 instances of Terrain for each hex. The terrains contains information about
    the terrain type (forest, hill etc), altitude and other modifiers.
    """

    def __init__ (self, xsize, ysize):
        "Initializes the map and allocates necessary data."
        # store the size
        self.size = ( xsize, ysize )

        # initialize the array of hexes
        self.hexes = [ None ] * ysize
        for index in range (ysize):
            self.hexes[index] = [ None ] * xsize

        # initialize the array of altitude formulas
        self.tris = [ None ] * ysize
        for index in range (ysize):
            self.tris[index] = [ None ] * xsize

        # now loop over the entire matrix and init the alts
        for y in range (ysize):
            for x in range (xsize):
                # create the tuple
                self.tris[y][x] = [ (0,0,0), (0,0,0), (0,0,0),
                                    (0,0,0), (0,0,0), (0,0,0) ]

        # a dict with all features of the map. we have nothing by default
        self.features = {}

        # a cache of filenames and icons
        self.filename_cache = None
        self.icon_cache = {}
        

    def getSize (self):
        "Returns the size of the map. The result is a tuple (x,y)."
        return self.size


    def getXsize (self):
        "Returns the x-size of the map."
        return self.size[0]


    def getYsize (self):
        "Returns the y-size of the map."
        return self.size[1]


    def getHexes (self):
        """Returns the 2D matrix of the hexes of the map."""
        return self.hexes


    def getHex (self, x, y):
        """Convenience method that returns a hex with given coordinates."""
        return self.hexes[y][x]


    def isInside(self, x, y):
        """Returns true if x,y coord is inside map."""
        #return x >= 0 and x < self.size[0] and y >= 0 and y < self.size[1]
        return 0 <= x < self.size[0]  and  0 <= y < self.size[1]


    def __getNeighbors__(self, hexx, hexy):
        if hexy % 2 == 0:
            return [(hexx,hexy-1,1),(hexx+1,hexy,2),(hexx,hexy+1,4),
                (hexx-1,hexy+1,8),(hexx-1,hexy,16),(hexx-1,hexy-1,32)]
        else:
            return [(hexx+1,hexy-1,1),(hexx+1,hexy,2),(hexx+1,hexy+1,4),
                (hexx,hexy+1,8),(hexx-1,hexy,16),(hexx,hexy-1,32)]


    def getNeighbors(self, hexx, hexy):
        """Returns valid neighbors as a list of (x, y, bit) tuples.
        The bit value starts at up-right, going clockwise."""
        # I need this implementation in ./src/editor/plugins/creator.py
        neighbors = self.__getNeighbors__(hexx, hexy)
        size = scenario.map.getSize()
        return [(hx,hy,bit) for (hx,hy,bit) in neighbors 
            if not (hx >= size[0] or hy >= size[1] or hx < 0 or hy < 0)]


    def hexHeight ( self, x, y):
        """center altitude of hex x,y - estimate if there is no such hex."""
        # really, we should average the real altitudes of adjacent hexes, if any have some,
        # for default, then fall back on using 0.

        print "Map.hexHeight: TODO: needs to be updated"

        altitude = 0
        try:
            altitude = self.getHex(x,y).getHeight()
        finally:
            return altitude


    def getHexCorners(self, x, y):
        """lists the map coordinates of the corners of the hex with hex
        coordinates (x,y), starting at the top and working clockwise."""
        px,py = self.hexToPoint([x,y])
        return [[px,py-27],[px+24,py-9],[px+24,py+9],[px,py+27],[px-24,py+9],[px-24,py-9]]


    def getHexAdjacent(self, hexx, hexy):
        """lists the hex coords adjacent to hex with hex coord x,y, starting
        at upper right and working clockwise."""
        if hexy % 2 == 0:
            return [(hexx,hexy-1),(hexx+1,hexy),(hexx,hexy+1),
                (hexx-1,hexy+1),(hexx-1,hexy),(hexx-1,hexy-1)]
        else:
            return [(hexx+1,hexy-1),(hexx+1,hexy),(hexx+1,hexy+1),
                (hexx,hexy+1),(hexx-1,hexy),(hexx,hexy-1)]


    def calculateHexTris ( self, x, y ):
        """ given hex coordinates (x,y) , calculate & return a list of the planes which compose
        its triangles (usual order, upper right then clockwise).
        Right now, return coefficients of plane (a,b,c) to save object overhead, not planes."""

        adjHex = self.getHexAdjacent(x,y)
        # first, calculate corners list [[x,y,z],[x,y,z]...
        corners = self.getHexCorners(x,y) # flat.
        for index in range(6):
            # average the 3 adjacent hex heights for each corner height
            height = (self.hexHeight(x,y) + self.hexHeight(adjHex[index][0], adjHex[index][1]) +
                      self.hexHeight(adjHex[(index-1) % 6][0],adjHex[(index-1) % 6][1]))/3.0
            corners[index].append(height)
        center = self.hexToPoint((x,y)) + [self.hexHeight(x,y)]
        tris = []

        # for the center and each pair of adjacent corners, calcuate plane that contains them.
        for index in range(6):
            print center, corners[index], corners[(index+1) % 6]
            slope = util.geometry.plane3d(center, corners[index], corners[(index+1) % 6])
            tris.append(slope.coeffs())
        return tris


    def getHeight (self, pos):
        """Returns the height of the map position pos, which is a (x,y) tuple. The position should
        refer to map coordinates, not hex coordinates. Looks up the height in the internal matrix of
        terrains."""

        # Find out the triangle the point is in.
        px, py = pos
        
        # Map the map coordinare to hexes
        hx, hy = self.pointToHex2(pos)
        
        # Map hex to map coords. Note! we get the _center_ of the hex
        cx, cy = self.hexToPoint((hx, hy))

        coefficient = 3 * pi
        triangle = int(floor( atan2(cx-px,py-cy) * coefficient ) % 6)
        # should work.  Or maybe it's exactly backwards... needs testing.
        (a,b,c) = self.tris [hy][hx][triangle]
        return a*px + b*py +c
    

    def getTerrain (self, pos):
        """Returns the terrain for position (x,y) tuple 'pos'. The passed parameter is given in map
        coordinates, *not* hexes."""

        #ick.  need a pointToHexTriangle function.  Wasted calculation.
        
        # map the map coordinare to hexes
        hx,hy = self.pointToHex2 ( pos )
         # find out the triangle the point is in.
        px, py = pos
        cx, cy = self.hexToPoint((hx, hy))
        coefficient = 3 * pi
        triangle = int(floor( atan2(cx-px,py-cy) * coefficient ) % 6)
        # should work.  Or maybe it's exactly backwards... needs testing.

        return self.hexes[hy][hx].getTerrain( triangle )


    def losValue (self, p1, p2, baseline):
        """Visibility from p1 to p2.  Less than 1 is not really visible; higher is more visible.
        This is where accounting for terrain, etc., should happen.  Speed of the current algorythm
        may be a problem."""
        x,y = p1
        z = self.getHeight((x,y)) + 3 # assume we're 3M tall. :>
        x2,y2 = p2
        z2 = self.getHeight((x2,y2)) + 3  # and so are they :>
        # inverse square.  This makes sense, really.  Round down, integerize.
        try:
            visibility = (baseline ** 2) / float((x2-x)**2+(y2-y)**2)
        except ZeroDivisionError: # p1 = p2 (!)  same location.
            return 1
        
        # LoSP is plane containing LoS vector, to calculate height.
        #  3rd point arbitrary, but we pick special not to be colinear with others.
        LoSP = util.geometry.plane3d((x,y,z),(x2,y2,z2),(x+y2-y,y+x-x2,0))

        # break LoS into crossings of individual triangles
        crossings = self.triangleCrossings( (p1,p2) )
        for index in range(len(crossings)-2):
            A, B = crossings[index:index+2]
            # because triangles are flat, max/min altitudes will be an edge.
            LoSaltitude = min(LoSP.value(A),LoSP.value(B))
            GroundAltitude = max(self.getHeight(A),self.getHeight(B))
            DistanceInHexes = sqrt((A[1]-B[1])**2+(A[0]-B[0])**2)/48.0 # 48 is length of hex
            MidPoint = ( ( (A[0]+B[0]) /2) , ( (A[1]+B[1]) /2) )
            visibility = visibility*(self.getTerrain(MidPoint).visionModifier(LoSaltitude-GroundAltitude)**DistanceInHexes)
            # not sure other code can handle vis other than 0 or 1 yet, so...
        visibility = int(floor(visibility))
        if visibility>1:
            visibility=1
        return visibility


    def pointToHex2 ( self, P):
        """New method to calculate in what hex point P is."""
        xp, yp = P
        # The calculations below are designed for icon at
        # hex coords (0,0) to be at gfx coords (-24, -12),
        # top-left corner of the icon.
        # Do a trivial translation here otherwise

        #xp -= 0
        #yp -= 0
        
        # Assume hexagon with two vertical lines and the rest diagonal
        width = 48
        height = 36
        
        magic = 12 # For horizontal region calculations
        # Base calculation of what x and y hex this is
        #
        # Yes, the int() is necessary because somebody
        # sends us floats...
        y = int(yp) / height
        x = int(xp) / width
        if y % 2 == 0 and xp % width >= width / 2:
            x += 1

        # Divide icon into three horizontal regions, of which the two first
        # are trivial, we're in a rectangle. Only one region requires
        # special attention, those with diagonal lines.
        if (yp / magic) % 3 != 2:
            # Easy case, we're in the "inside rectangle" of a hex
            # Do nothing, base calculation is correct
            pass
        else:
            # xh value:  0  24  0  
            # \    /\    /\    /   0
            #  \  /  \  /  \  /         yh value
            #   \/    \/    \/     12
            xh = xp % width
            yh = yp % magic
            if xh > width/2:
                xh = width - xh
            if y % 2 == 0:
                # We're in a /\ group
                # Invert xh
                xh = width/2 - xh
                # On which side of the hex lines are we?
                if width / 2 * yh > xh * magic:
                    y += 1
                    # We could have done a +1 higher up in base
                    # calculations, which is wrong. So recalculate
                    x = xp / width
            else:
                # We're in a \/ group
                # xh is already correct.
                # On which side of the hex lines are we?
                if width / 2 * yh > xh * magic:
                    y += 1
                    if xp % width > width/2:
                        x += 1

        # NOTE: We don't require it to be within
        # bounds of the map size, since various
        # routines (like MOUSEMOVE events) need to check
        # for out-of-bounds hexes. Hmm.. perhaps we
        # could throw some kind of an Exception
        return ( x,y )

##    def pointToHex ( self, P ):
##        """ given point, return hex coordinates of that point's hex """
##        center = self.pointToHexCenter(P)
##        Y = center[1] / 36
##        if Y % 2 == 0:
##            return ( int ( center[0] / 48) , int (Y) )
##        else:
##            return ( int ( ((center[0]-24) / 48) ), int (Y) )


##    def pointToHexCenter ( self, P ):
##        """Returns center of hex that contains point P.
##        We chop all the hexes into triangles with 3 sets of parallel
##        lines, find out what triangle it's in, one of the points of
##        that is also the center of the hex it's in.  There ought to be
##        a simpler way to do this..."""
##        x = P[0]
##        y = P[1]
##        # currently assumes hexsizes, fix this later.
##        #  we use floor+1, rather than ceil, because we want
##        #  two lines even if the point is on one exactly.
##        A1 = util.geometry.line2d ( 1, 0, ( floor(x/24.0) * 24 ))
##        A2 = util.geometry.line2d ( 1, 0, ( (floor(x/24.0)+1) * 24 ))
##        B1 = util.geometry.line2d ( 1, 2, ( floor((x+2*y)/48.0) * 48 ))
##        B2 = util.geometry.line2d ( 1, 2, ( (floor((x+2*y)/48.0)+1) * 48 ))
##        # one of these four points is the center of our hex
##        S1 = [ A1.intersect(B1), A1.intersect(B2), A2.intersect(B1), A2.intersect(B2) ]
##        S2 = []
##        for v in S1:
##            if v[1] % 36 == 0:  # hex centers are the only triangle vertexes 0 mod 36!
##               S2.append(v) # only keep points which are the centers of hexes
##        if len(S2) == 1:
##            return S2[0]
##        elif len(S2) == 2: # two were hex centers.  Which is on the other lines?
##            C1 = util.geometry.line2d ( 1, -2, ( floor((x-2*y)/48.0) * 48 ))
##            C2 = util.geometry.line2d ( 1, -2, ((floor((x-2*y)/48.0)+1) * 48 ))
##            if C1.hasPoint(S2[0]) or C2.hasPoint(S2[0]):
##                return S2[0]
##            elif C1.hasPoint(S2[1]) or C2.hasPoint(S2[1]):
##                return S2[1]
##            else:
##                raise "impossible point"+`P` # mikee thinks this can't happen.

            
    def hexToPoint (self, hex):
        """Returns a (x,y) tuple that contains the center of the given hex in global coordinates."""
        # optimization. avoids dots
        hexsize     = properties.hex_size
        deltax      = properties.hex_delta_x
        deltay      = properties.hex_delta_y
        deltax_half = deltax / 2

        # explode the hex
        x, y = hex

        # This catches most guys who pass in map coords instead of hex coords
        assert(self.isInside(x, y))
        
        # now determine wether we have a odd or even offset and the current row. This is a
        # little bit ugly, but it works. Some artifacts remain on the edges though
        if ( y ) % 2 == 0:
            # the icon is painted as far left as possible
            return x * deltax - deltax_half, y * deltay - (hexsize - deltay ) 

        # the icon is indented half a hex to the right.
        return x * deltax, y * deltay - (hexsize - deltay )


    def triangleCrossings (self, V):
        """V is a vector of two points
        Returns the list of points where V crosses triangle boundries
         and the points themselves."""
        P1, P2 = V
        x1, y1 = P1
        x2, y2 = P2
        path = util.geometry.line2dFromPoints(P1,P2)
        Crossings = [P1, P2]
        # probably not fastest way to do this.
        first = ceil(min((x1/24.0),(x2/24.0)))
        last = ceil(max((x1/24.0),(x2/24.0)))
        for index in range(first, last):
            L = util.geometry.line2d ( 1, 0, ( index * 24 ) ) 
            Crossings.append(L.intersect(path))
        first = ceil(min(((x1+2*y1)/48.0),((x2+2*y2)/48.0)))
        last = ceil(max(((x1+2*y1)/48.0),((x2+2*y2)/48.0)))
        for index in range(first, last):
            L = util.geometry.line2d ( 1, 2, ( index * 48 ))
            Crossings.append(L.intersect(path))
        first = ceil(min(((x1-2*y1)/48.0),((x2-2*y2)/48.0)))
        last = ceil(max(((x1-2*y1)/48.0),((x2-2*y2)/48.0)))
        for index in range(first, last):
            L = util.geometry.line2d ( 1, -2, ( index * 48 ))
            Crossings.append(L.intersect(path))
        Crossings.sort()
        result = [Crossings[0]] 
        for i in range(1,len(Crossings)):
            if Crossings[i]!=Crossings[i-1] and Crossings[i] != []:
                result.append(Crossings[i])
        return result


    def getAllFeatures (self):
        """Returns all features that have been loaded. If no features exist in the map then an empty
        dictionary is returned. The dict is indexed by the type of feature, such as 'house',
        'trench' etc, and the value for each key is a list of feature tuples. Each tuple contains
        tha data (x, y, id, icon)."""
        return self.features
    

    def addFeature (self, type, id, x, y):
        """Adds a new feature to the internal dictionary. A feature is something that is blitted on
        top of the normal terrain, and just adds to it. The 'type' is used to organize the data in
        the internal dict. The position defines where the feature is (upper left corner). The id is
        the unique id of the feature's icon."""

        # do we already have such features?
        if not self.features.has_key ( type ):
            # nope, add an empty list
            self.features [type] = []

        # now we want to load an icon too if we're a graphical client, ie non-ai
        if scenario.local_player_ai:
            # local player is ai, no icon needed
            icon = None
        else:
            # load the icon
            icon = self.loadFeatureIcon ( id )
        
        # add the data about this feature 
        self.features [type].append ( (x, y, id, icon) )


    def loadFeatureIcon (self, id):
        """ """
        # do we already have loaded the icon?
        if self.icon_cache.has_key ( id ):
            # yes, return it
            return self.icon_cache[id]

        
        # not yet loaded the icon, so do we already have a cache?
        if self.filename_cache == None:
            # no, so get a list of all files in the features directory
            allFiles = os.listdir ( properties.path_features )
        
            # create the regular expression for matching the id in the name
            iconExp = re.compile ( '^t-.+([0-9]{3}).png$' )

            # init the cache to an empty dict
            self.filename_cache = {}
            
            # loop over all files in the directory
            for file in allFiles:
                # attempt to match the expression
                match = iconExp.search ( file )

                # did it match?
                if match:
                    # yep, set it in the temporary dictionary
                    self.filename_cache [ int (match.group (1)) ] = properties.path_features + file

                    print "Map.loadFeatureIcon: found:", properties.path_features + file
                    
        # we do have a cache at least now
        try:
            filename = self.filename_cache [id]
        except:
            # oops, no such id?
            raise "Map.loadFeatureIcon: no feature icon with id %d found" % id

        # load the icon
        icon = pygame.image.load ( filename )
    
        # set the transparent color
        icon.set_colorkey ( (255,255,255), RLEACCEL )

        # The try block is because editor doesn't use sdl
        try:
            # convert the icon to a more efficient format
            icon = icon.convert ()
        except:
            pass

        # store the icon in ourt cache
        self.icon_cache [id] = icon

        return icon


    def isErrorSquare(self, x, y, thelist):
        """Returns true if x,y is in the error list.
        Note that the amount of elements in thelist will change, so I
        did this convenience function instead..."""
        return (x, y) in thelist
    
##         for i in thelist:
##             if x == i[0] and y == i[1]:
## 	        return 1
##         return 0


    def __calcHeight__(self, hexx, hexy, dir, errorlist):
        """Calculate what height we should have at (hexx,hexy),
        depending on the hex in direction dir."""
        thehex = self.__getNeighbors__(hexx, hexy)[dir-1]
        size = scenario.map.getSize()
        if thehex[0] < 0 or thehex[0] >= size[0] or thehex[1] < 0 or thehex[1] >= size[1]:
            return None
        checkhex = self.getHex(thehex[0], thehex[1])
        checkheight = checkhex.getHeight()
        oppositedir = (3+dir-1) % 6 +1
        checkheight += checkhex.template.getDeltaHeight(oppositedir)
        # And add the negative delta height of the next hex
        checkheight -= self.getHex(hexx, hexy).template.getDeltaHeight(dir)
        if self.isErrorSquare(thehex[0], thehex[1], errorlist):
            return (0, checkheight)
        return (1, checkheight)
        

    def calculateAbsoluteHeights (self):
        """This method will loop over all hexes and calculate an overall height for all the
        triangles of all hexes. When loaded the heights are all 0. This method starts from one
        corner of the map and propagates the height to neighbours to get an overall and absolute
        height. When the heights are calculated an offset is added so that the lowest point on the
        map has the height 0.

        This should be called after all hexes have been set."""
        print "Map.calculateAbsoluteHeights"

        height = 0
        lowest_height = 0

        # list of (x,y) pairs of map height errors
        # this way we can throw all errors at the same time
        map_errors = []
        
        for y in range(self.size[1]):
            for x in range(self.size[0]):
                curhex = self.getHex(x, y)
                curhex.setHeight(height)

                # update lowest height
                lowest_height = min(lowest_height, height)

                # Calculate height for next hex
                if x == self.size[0]-1:
                    if y == self.size[1]-1:
                        # last hex
                        break
                    # right edge
                    # Skip to next row
                    height = self.__calcHeight__(0, y+1, 1, [])[1]
                else:
                    heights = []

                    h1 = self.__calcHeight__(x+1, y, 1, map_errors)
                    h2 = self.__calcHeight__(x+1, y, 6, map_errors)
                    h3 = self.__calcHeight__(x+1, y, 5, map_errors)
                    if h1 and h1[0]:
                        heights.append(h1[1])
                    if h2 and h2[0]:
                        heights.append(h2[1])
                    if h3 and h3[0]:
                        heights.append(h3[1])

                    if len(heights) == 0:
                        # Everything bad, we can just take one of them
                        if h1:
                            height = h1[1]
                        elif h2:
                            height = h2[1]
                        elif h2:
                            height = h3[1]
                        map_errors.append((x+1, y))
                    else:
                        best_index = 0
                        best_times = -1
                        for i in range(len(heights)):
                            times = heights.count(heights[i])
                            if times > best_times:
                                best_index = i
                                best_times = times
                        height = heights[best_index]
                        if best_times != len(heights):
                            map_errors.append((x+1, y))
                                    

        if map_errors:
            print "Map height errors found."
            print map_errors
            raise MapHeightException(map_errors)

        else:
            print "No map errors found."

        # Update heights so lowest height on map is zero.
        if lowest_height != 0:
            for y in range(self.size[1]):
                for x in range(self.size[0]):
                    curhex = self.getHex(x, y)
                    h = curhex.getHeight()
                    h -= lowest_height
                    curhex.setHeight(h)


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
