###############################################################################################
# $Id: messages.py,v 1.16 2002/05/08 11:16:39 chakie Exp $
###############################################################################################

import sys
import time
import scenario
import properties
import pygame
import pygame.image
from pygame.locals import *

# some constants used for colorizing the messages
NORMAL        = 0
CHAT1         = 1
CHAT2         = 2
COMBAT        = 3
DESTROYED     = 4
REINFORCEMENT = 5
ERROR         = 6
AUDIO         = 7
HELP          = 8

class Messages:
    """
    This class keeps track of all messages that the other player or the engine have sent to the
    local player. The messages are normal textual strings that contain some for if textual
    information meant for the player.

    The messages are kept as prerendered surfaces, ready for blitting.
    """

    def __init__ (self):
        """Creates the instance of the class. Resets all data."""
        # no labels yet
        self.labels = []

        # create the colors for the various types of messages
        self.colors = { NORMAL:        (255, 255, 255),
                        CHAT1:         (255, 255, 0),
                        CHAT2:         (0, 255, 255),
                        COMBAT:        (128, 255, 255),
                        DESTROYED:     (135, 206, 250),
                        REINFORCEMENT: (255, 128, 128),
                        ERROR:         (0, 255, 255),
                        AUDIO:         (0, 255, 0),
                        HELP:          (210, 210, 255) }
        
        # get the font we should use
        self.font = pygame.font.Font ( properties.layer_messages_font_name,
                                       properties.layer_messages_font_size )

        # get the shadow color
        self.shadowcolor = properties.layer_messages_shadow_color
        
        # get the max width of labels
        self.width = properties.layer_messages_max_width

        # register ourselves with the animation manager. we want to be called regularly, but not too
        # frequently
        scenario.animation_manager.register ( properties.messages_animation_interval, self.animate )


    def animate (self):
        """Callback executed by the animation manager when it's time to perform animation. Actually
        it's no real animation, but just a way to make sure the messages on the playfield silently
        scroll away."""

        # ok, do we even have messages here?
        if len (self.labels) == 0:
            # no messages, get lost
            return

        # get the current millisecond
        millisecond = scenario.time.getTicks ()

        # should the first label be removed? we allow them at least 20s on display
        if millisecond - self.labels [0][0] < 20000:
            # too young, don't remove yet
            return
        
        # just slice away the first message, this removes it
        self.labels = self.labels [1:]

        # The messages layer catches this and repaints
        scenario.dispatcher.emit ( 'messages_changed', None )
 
        
    def getLabels (self):
        """Returns the prerendered labels, ready for blitting. The text in the messages can no
        longer be obtained from the labels."""
        return self.labels


    def add (self, message, type=NORMAL):
        """Adds a new message. The message is internally stored as a surface ready for blitting
        along with the hour and minute (game time) that tell when the message was received. The
        parameter 'type' indicates the type of message. Various types can have different colors to
        make it easier for the player to visualize the messages. The parameter defaults to NORMAL
        color, which can be used for anything. See the beginning of this file for the other
        types."""

        # get the hour and minute and second (the second is ignored)
        hour, minute, second = scenario.time.getTime ()

        # get the current millisecond
        millisecond = scenario.time.getTicks ()

        tmp = ''
        all = ''

        # get the color we should use
        color = self.colors [ type ]
        
        # loop over all words
        for text in message.split ( ' ' ):
            # store the current full line
            tmp = all

            # merge a text we use to test the width with. Don't add a ' ' if we only have one word
            # so far
            if all == '':
                test = text
            else:
                test =  all + ' ' + text

            # get the size of the label as it would be when rendered
            x, y = self.font.size ( test )

            # too wide?
            if x > self.width:
                # yep, os use the last 'good' text that fits and render a label
                self.labels.append ( (millisecond, self.renderLabel ( all, color ) ))

                # start with a new full line that is the part that was 'too much'
                all = text
            
            else:
                # it's ok, append new text to full line
                all = all + ' ' + text 
        
        # still something in 'all' that has not made it into a full line? we add a last (short) line
        # with the extra text
        if all != '':
            self.labels.append ( (millisecond, self.renderLabel ( all, color ) ))
            
        # do we have too many labels?
        if len ( self.labels ) > properties.messages_max_labels:
            # we need to get rid of a few labels. we slice away as many labels as are too
            # many. simple and efficient
            self.labels = self.labels [ len (self.labels) - properties.messages_max_labels: ]

        # The messages layer catches this and repaints
        scenario.dispatcher.emit ( 'messages_changed', None )

 
    def renderLabel (self, text, color):
        """Renders the given text using the given color. The label will have a small shadow around
        it to make it easier to read and give better contrast. Or something. The shadow is created
        by first blitting the shadow four times, offset a little bit, and then finally the main
        text. """

        # first render the main text and a shadom
        maintext = self.font.render ( text, 1, color )
        shadow   = self.font.render ( text, 1, self.shadowcolor )

        # now we get the size of that surface, as we want a slightly larger final surface
        width  = maintext.get_width () + 3
        height = maintext.get_height () + 3

        # now allocate the final surface
        label = pygame.Surface ( ( width, height ), HWSURFACE )

        # fill with a color we can make transparent later
        label.fill ( (10,10,10) )

        for x, y in ( (0,0), (1,0), (2,0), (0,1), (2,1), (0,2), (1,2), (2,2) ):
            # blit out the outline offset a few pixels
            label.blit ( shadow, ( x, y ))
       
        # now blit out the actual text in the middle of the surface
        label.blit ( maintext, ( 1, 1 ))

        # make blackish pixels transparent
        label.set_colorkey ( (10, 10, 10) )

        # we're done, return our nice label
        return label
    
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
