###############################################################################################
# $Id: scenario_info.py,v 1.22 2002/09/05 19:21:04 msa Exp $
###############################################################################################

import sys
import re
import scenario
from organization import REBEL, UNION

class ScenarioInfo:
    """
    This class contains all needed information about the current scenario. The scenario is the
    current battle, and the information contained is the name, a description, the historical date
    and the number of turns in the battle.

    The description and the mission attribute is a list of strings, each string representing one
    paragraph. 

    This class basically reflects the tag <scenarioinfo> in the scenario files.
    
    Methods exist for accessing all the information.
    """

    nextid = 0
    
    def __init__ (self, id=None):
        """Creates the instance of the class."""
        # init all members
        self.name        = ""
        self.description = []
        self.maxturns    = 0
        self.currentturn = 0
        self.date        = ()
        self.location    = ""

        # all brigades in the game. THe brigades then in turn contain all other organization, such as
        # regiments, battallions and companies in a tree-like structure. The map has the id of the unit as a
        # key 
        self.brigades = { REBEL: {},
                          UNION: {} }

        # all companies and batteries of the game. These can be reached from the 'Brigades' instance as well,
        # but usually only the companies/batteries need to be accessed, so this structure is provided for
        # convenience. The map has the unit id as a key
        self.units = {}

        # a map with all destroyed units. This works in the same way as 'units' above does, with the
        # exception that only units which have been destroyed are here. So companies that are here are *not*
        # in 'units' and vice versa. the units are indexed by their id

        self.destroyed_units = {}

        # a hash that maps unit id:s to a tuple (unit,turn) which is the actual unit and the turn it arrives
        # Turn <= 0 indicates that they are present there when the battle starts
        self.reinforcements = {}

        # a list of all objectives on the map
        self.objectives = []

        # a list of all locations
        self.locations = []

        # did we get an id manually?
        if id != None:
            # yep, use it
            self.id = id
        else:
            # use the internal unique id generator
            self.id = ScenarioInfo.nextid
            ScenarioInfo.nextid += 1
        
        # set the default missions
        self.missions = { REBEL: [],
                          UNION: [] }
        

    def getId (self):
        """Returns the unique id of the instance."""
        return self.id


##     def setId (self, id):
##         """Set a new id for the instance."""
##         self.id = id
    

    def getName (self):
        """Returns the name of this scenario."""
        return self.name


    def setName (self, name):
        """Sets a new name for the scenario."""
        self.name = name


    def getLocation (self):
        """Returns the location of this scenario."""
        return self.location


    def setLocation (self, location):
        """ Sets a new location for the scenario."""
        self.location = location


    def getDescription (self):
        """Returns the description of this scenario. This returns a list which contains strings, one
        for each paragraph. The list is empty if there is no description."""
        return self.description


    def setDescription (self, description):
        """Sets a new description for the scenario. The parameter should be a list of strings."""
        self.description = description


    def getMission (self, player):
        """Returns the mission description for the given 'player'. The mission description is like
        the overall description a list with strings, each string a paragraph of text. It may be
        empty too."""
        # just brutally assume the passed player is ok
        return self.missions [ player ]


    def setMission (self, player, description):
        """Sets a new mission description for 'player'.  The mission description is like
        the overall description a list with strings, each string a paragraph of text. It may be
        empty, i.e. an empty list."""
        # precautions
        if not player in ( REBEL, UNION ):
            # ouch, fubar player
            raise ValueError ( "invalid player id given" )
        
        # just set it
        self.missions [ player ] = description


    def getCurrentTurn (self):
        """Returns the current turn of this scenario. This can be used when loading a saved scenario
        to get the turn where the game stopped when it was saved."""
        return self.currentturn


    def setCurrentTurn (self, turn):
        """Sets a new current turn for the scenario."""
        self.currentturn = turn
        

    def getMaxTurns (self):
        """Returns the maximum number of turns of this scenario."""
        return self.maxturns


    def setMaxTurns (self, maxturns):
        """Sets a new value of the maximum turns for the scenario."""
        self.maxturns = maxturns
        

    def getDate (self):
        """Returns the date of this scenario. This is a (year,month,day,hour,minute) tuple."""
        return self.date


    def setDate (self, date):
        """Sets a new date for the scenario. This must be a (year,month,day,hour,minute) tuple"""
        self.date = date


    def toXML (self):
        """Returns a string containing the scenario info serialized as XML. The toplevel tag is
        <scenarioinfo>."""

        # build up the xml. This is really quite nice, I'm impressed with Python's handling of strings!
        xml  = '\t<scenarioinfo>\n'
        xml += '\t\t<name>%s</name>\n'                % self.name 
        xml += '\t\t<turns current="%d" max="%d"/>\n' % ( self.currentturn, self.maxturns )
        xml += '\t\t<location>%s</location>\n'        % self.location

        # do we have a local player? this one should be set if we're saving from within a game, but
        # should not be set in the editor
        if scenario.local_player_id != None:
            # yes, we have one. add it
            xml += '\t\t<localplayer id="%d"/>\n' % scenario.local_player_id

        # write out all other data
        xml += '\t\t<date year="%d" month="%d" day="%d" hour="%d" minute="%d"/>\n' % self.date
        xml += "\t\t<description>\n"
        
        # and the description. First, do we have one at all?
        if self.description == []:
            # no description
            xml += "\t\t</description>\n"

        else:
            # loop over all the paragraphs
            for para in self.description:
                # add the para
                xml += "\t\t\t<para>%s</para>\n" % para

            # close the description
            xml += "\t\t</description>\n"

        # now the missions
        xml += "\t\t<missions>\n"

        # loop over them
        for label, player in ( ('rebel', REBEL), ('union', UNION ) ):
            # get the description
            text = self.getMission ( player )

            # print the start
            xml += "\t\t\t<%s>\n" % label
            
            # loop over all the paragraphs
            for para in text:
                # add the para
                xml += "\t\t\t\t<para>%s</para>\n" % para

            # print the end
            xml += "\t\t\t</%s>\n" % label
            
        
        xml += "\t\t</missions>\n"
        xml += "\t</scenarioinfo>\n"

        return xml

    def setupScenario (self):
        """Performs various scenario setup. When this method is called the scenario is already
        loaded and the players have been set up. Currently does:

        * set visibility of units
        * store id of local player
        * calculate overall height of the map
        """

        # loop over all units
        for unit in self.units.values ():
            # set proper visibility on the unit.
            unit.calcView()

        # calculate map absolute heights
        scenario.map.calculateAbsoluteHeights ()
        



#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
