#! /usr/local/bin/python
###############################################################################################
# $Id: scenario_manager.py,v 1.26 2002/05/13 08:50:37 chakie Exp $
###############################################################################################

import os, re, sys, string
import os.path
import pyexpat
import gzip

import scenario
import properties
from simple_dom_parser import SimpleDOMParser, Node
from scenario_info     import ScenarioInfo
from organization      import REBEL, UNION

class ScenarioManager:
    """
    This class works as a base class for managing scenarios in Civil. The subclasses
    ClientScenarioManager and ServerScenarioManager are those that should be instantiated by the
    client and server. This class contains common functionality needed by both classes.
    """

    def __init__ (self):
        """Intializes the instance."""
        # init all members
        self.scenarios = []

        # a map from id:s to scenario filenames
        self.filenames = {}
        
    
    def getScenarios (self):
        """Returns all the parsed scenario infos. The returned list contains instance of
        ScenarioInfo that describe all the scenarios that are available. If something has failed or
        there are no scenarios available the list is empty."""
        return self.scenarios
    

    def parseScenarioInfo (self, root):

        # is it a proper index file?
        if root.getName () != 'scenarioinfo':
            # oops, illegal file
            print "DOM tree is illegal, toplevel tag is: " + root.getName () + ", "
            print "the node should be 'scenarioinfo'"
            return None

        try:
            # get the simple data
            name        = root.getChild ('name').getData ()
            location    = root.getChild ('location').getData ()
            maxturns    = int ( root.getChild ('turns').getAttribute ( 'max' ) )
            currentturn = int ( root.getChild ('turns').getAttribute ( 'current' ) )
 
            # get the file if we have it
            file = root.getChild ('file')
                
            # get the date and it
            datenode = root.getChild ('date')
            year     = int ( datenode.getAttributes ()['year'] )
            month    = int ( datenode.getAttributes ()['month'] )
            day      = int ( datenode.getAttributes ()['day'] )
            hour     = int ( datenode.getAttributes ()['hour'] )
            minute   = int ( datenode.getAttributes ()['minute'] )

            # no description yet
            description = []
            
            # and the description
            for paratag in root.getChild ('description').getChildren ():
                # get the text
                para = paratag.getData ()
                
                # append the text to the description
                description.append ( para )

            # get the rebel and union missions
            rebel = []
            union = []

            # get the missions
            mission = root.getChild ('missions')
            
            # rebel misisons
            for paratag in mission.getChild ( 'rebel').getChildren ():
                # get the text and append to the description
                rebel.append ( paratag.getData () )

            # union misisons
            for paratag in mission.getChild ( 'union').getChildren ():
                # get the text and append to the description
                union.append ( paratag.getData () )
            
            # create a new scenario info and set all parsed data
            info = ScenarioInfo ()
            info.setName ( name )
            info.setLocation ( location )
            info.setMaxTurns ( maxturns )
            info.setCurrentTurn ( currentturn )
            info.setDescription ( description )
            info.setMission ( REBEL, rebel )
            info.setMission ( UNION, union )
            info.setDate ( (year, month, day, hour, minute ) )

            # add to our internal list
            self.scenarios.append ( info )
            
            # all ok
            return info
        
        except KeyError:
            # oops, invalid data
            print "Invalid scenario info."
            return None
    

##################################################################################################

class ClientScenarioManager (ScenarioManager):
    """This class subclasses the scenario manager and adds methods needed by the clients to manage
    scenarios. The methods are those used to communicate with the server and request/retrieve
    scenarios. """
    

    def __init__ (self):
        """Initializes the instance."""
        # call superclass
        ScenarioManager.__init__ (self )

        
    def parseScenarioIndex (self, root):
        """Parses a passed DOM-tree and builds up a number of instances of ScenarioInfo. They are
        appended to the internal list of senario-info:s. Returns 1 if all is ok and the entrie index
        file was parsed, and 0 on error."""

        # is it a proper index file?
        if root.getName () != 'scenarioindex':
            # oops, illegal file
            print "DOM tree is illegal, toplevel tag is: " + root.getName () + ", "
            print "the node should be 'scenarioindex'"
            return 0
        
        # loop over all scenarioinfo:s
        for info in root.getChildren ():

            # parse the current info
            if not self.parseScenarioInfo ( info ):
                # parsing failed
                print "parsing failed"
                return 0

        # all is ok
        return 1
    
    
    def retrieveScenarios (self, connection):
        """Reads scenarios from the server over open connection. Will request the server to send all
        the available scenarios to us. Not the full scenarios are sent, only the info part
        (header). This information is all the player needs to be able to decide upon which scenario
        to use."""

        # request the scenarios
        connection.send ( 'get_scenarios\n' )
        
        try:
            # read the number of lines, wait for 5 seconds
            line = connection.readLine ( timeout = 5 )

            # did we get a line?
            if not line:
                print "ScenarioManager.retrieveScenarios: error or timeout receiving line count"
                return 0
           
            # cast the data
            count = int ( string.strip (line ) )
            xml_data = []
            
            # loop and read the lines
            for index in range ( count ):
                # read a line
                line = connection.readLine ( timeout = 5 )

                # did we get a line?
                if not line:
                    continue
                
                # add to the internal list
                xml_data.append ( line )

            # join the data
            buffer = ''
            buffer = '\n'.join ( xml_data )

            # parse the read data
            root = SimpleDOMParser ().parseString ( buffer )

        except:
            # serios error
            print "ScenarioManager.retrieveScenarios: error retrieving scenarios"
            raise
            return 0
                
        # did we get anything?
        if root == None:
            # nope, failed to retrieve it
            print "ScenarioManager.retrieveScenarios: failed to retrive scenarios"
            return 0

        # now parse the data
        self.parseScenarioIndex ( root )


    def retrieveFullScenario (self, connection, id):
        """Retrieves the full scenario 'id' from the server. Builds up a buffer with the data, but
        does not actually parse it in any way. Returns the buffer, or None on error."""

        # request the scenario
        connection.send ( 'scenario %d\n' % id )

        # no data yet
        buffer = ''
       
        try:
            # read the number of lines, wait for 5 seconds
            line = connection.readLine ( timeout = 5 )

            # did we get a line?
            if not line:
                print "ClientScenarioManager.retrieveFullScenario: error or timeout receiving line count"
                return 0
           
            # cast the data
            count = int ( string.strip (line ) )
            
            # loop and read the lines
            for index in range ( count ):
                # read a line
                line = connection.readLine ( timeout = 5 )

                # did we get a line?
                if not line:
                    continue
                
                # add to the internal list
                buffer += line

            # all is ok, return the buffer
            return buffer
 
        except:
            # serios error
            print "ClientScenarioManager.retrieveFullScenario: error retrieving scenario"
            raise
            return None


    def readScenario (self, connection, count):
        """Reads one scenario from 'connection'. This differs from the above the method
        retrieveFullScenario() as it does not first request the scenario, instead it assumes a
        scenario is already coming down the pipe. The scenario must be 'count' lines of
        data. Returns the buffer, or None on error.""" 

        buffer = ""

        try:
            # loop and read the lines
            for index in range ( count ):
                # read a line
                line = connection.readLine ( timeout = 5 )

                # did we get a line?
                if not line:
                    continue
                
                # add to the internal list
                buffer += line

            # all is ok, return the buffer
            return buffer
 
        except:
            # serios error
            print "ClientScenarioManager.readScenario: error retrieving scenario"
            raise
            return None


##################################################################################################

class ServerScenarioManager (ScenarioManager):
    
    """This class subclasses the scenario manager and adds methods needed by the server to manage
    scenarios. It mainly knows where to find information about all available scenarios and can then
    parse a wanted scenario file. Scenarios are quite big, so separate information about them is
    kept in an index files. This class knows where all the index files are, reads and parses them
    into a list with instances of ScenarioInfo. This list can then be accessed by external classes
    and the user can be presented with a list of all scenarios.

    The client class ClientScenarioManager will request scenarioinfos and scenarios from this
    class.

    This class can also write out a new scenario index file from all loaded scenarios.
    """

    def __init__ (self):
        """Initializes the instance and load the scenario index file."""

        # call superclass
        ScenarioManager.__init__ (self )

        
    def sendScenarioInfo (self, connection):
        """Sends the raw ASCII XML data to the client behind 'connection'. First the number of lines
        is sent and then the raw data one line at a time."""

        # build up the raw xml we're about to send
        xml = [ '<?xml version="1.0"?><scenarioindex>\n' ]

        # loop over all infos we got
        for info in self.scenarios:
            # add the xml for this info
            for line in info.toXML ().split ( '\n' ):
                xml.append ( line + '\n' )

        # append the closing tag
        xml.append ( '</scenarioindex>\n' )
        
        # get the number of lines
        count = str ( len ( xml ) ) + '\n'

        # send the number of lines
        connection.send ( count )
        
        # now loop over all lines
        for line in xml:
            # send it
            connection.send ( line )


    def sendScenario (self, connection, id):
        """Sends the scenario with the given id to the client to which 'connection' leads. First
        the number of lines, the id for the remote player and the name of the local player is sent
        and then the lines one at a time.""" 

        # map the id to a filename
        if not self.filenames.has_key ( id ):
            # no matching scenario found
            print "invalid scenario id:", id
            connection.send ( 'invalid_id\n' )

            return 

        # get the filename
        filename = self.filenames [id].replace ( '-info.xml', '.xml.gz' )

        # does the file exists?
        if not os.path.exists ( filename ):
            # no such file
            print "dangling info file, no main scenario file for: %s" % filename
            return

        #print "loading scenario: %s" % filename

        # read all lines
        lines = gzip.GzipFile ( filename, 'rb').readlines ()

        # send the basic data
        connection.send ( '%d %d %s\n' % ( (UNION, REBEL)[scenario.local_player_id],
                                           len ( lines ), scenario.local_player_name ) )
        
        # now loop over all lines
        for line in lines:
            # send it
            connection.send ( line )
            
        # we're done here
        return


    def loadScenario (self, id):
        """Loads the scenario with the given id. The scenario is returned as a list of strings, one
        line in each string. On error None is returned."""

        # map the id to a filename
        if not self.filenames.has_key ( id ):
            # no such id?
            print "invalid scenario id:", id
            return None

        # get the filename
        filename = self.filenames [id].replace ( '-info.xml', '.xml.gz' )

        # does the file exists?
        if not os.path.exists ( filename ):
            # no such file
            print "dangling info file, no main scenario file for: %s" % filename
            return None

        print "loading scenario: %s" % filename

        # read all lines
        scenario = gzip.GzipFile ( filename, 'rb' ).readlines ()
        
        # we're done here
        return scenario


    def loadScenarioInfo (self, filename):
        """This method loads an scenario info file. All new infos that can be parsed from the file
        are appended to the internal list of scenario infos. No checks are made for
        duplicates. Returns 1 if all is ok and 0 on error."""

        # does the info file exist?
        if not os.path.exists ( filename ):
            # no such file
            print "can not parse: %s does not exist. Please recreate the file" % filename
              
        # create a parser
        parser = SimpleDOMParser ()
            
        # parse the data
        try:
            # read in the raw data 
            xml = open ( filename ).readlines ()
            
            # loop over all lines join the data
            buffer = ''
            buffer = buffer.join ( xml )

        except:
            # serios error
            print "ServerScenarioManager.loadScenarioInfo: error loading index from %s" % filename
            return 0
            
        # parse the read data
        root = parser.parseString ( buffer )
                
        # did we get anything?
        if root == None:
            # nope, failed to retrieve it
            print "ServerScenarioManager.loadScenarioInfo: parse error for: " + filename
            return 0

        # now parse the data
        info = self.parseScenarioInfo ( root )

        # did we do ok?
        if info != None:
            # yes, all ok, store an entry in the name map so that we can later identify the file for
            # the info based on its id
            self.filenames [ info.getId () ] = filename

        # return whatever we got
        return info


    def loadAllScenarioInfos (self, dir):
        """Attempts to load all info files for scenarios that are found in 'dir'. Loads scenario
        infos from the files and stores them internally. Returns 1 if all is ok, and 0 if not."""
        
        # first get all files in the directory
        allfiles = os.listdir ( dir )
 
        # create a regexp for matching jpg:s
        exp = re.compile ( '-info.xml$' )

        # loop over all files
        for file in allfiles:
            # does this file match the pattern?
            if exp.search ( file ):
                # yep, parse it
                filename = dir + '/' + file 
                print "Parsing: %s..." % filename

                if not self.loadScenarioInfo ( filename ):
                    # we failed to read it, so abort all reading
                    return 0

        # we got this far, so all must be ok
        return 1


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
