###############################################################################################
# $Id: server_init_loop.py,v 1.5 2002/09/04 16:44:36 msa Exp $
###############################################################################################

import sys
import scenario
import server.properties
from string           import strip, join
from select           import select
from player           import Player
from net.connection   import Connection
from scenario_parser  import ScenarioParser
from organization     import UNKNOWN

#################################################################################################
def waitForData (clients, sock):
    """Waits for data on the connections to the clients and the incoming socket. The incoming socket
    is only used if only 0 or 1 players have connected. When both players have connected the socket
    is closed and not used anymore.

    Returns a tuple with the client connections which have data to be read _and_ all clients. If a
    new client arrives on the incoming socket it is accepted and a new Player is created and added
    to the list of new clients. Will wait forever, so this function should not be called unless
    there *will* come some data back."""

    connections = []

    # loop over all clients
    for client in clients:
        # precautions
        if not client.getConnection ().isValid ():
            # oops, something has happened
            clients.remove ( client)
        else:
            # add the connection
            connections.append ( client.getConnection () )

    # add the incoming socket too
    connections.append ( sock )
    
    # perform the select. We're only interested in incoming events. wait forever
    incoming, out, execptional = select ( connections, [], [] )

    # now we have the connections, but we want the clients, so check them both
    ready = []

    # loop over all clients
    for client in clients:
        if client.getConnection () in incoming:
            ready.append ( client )

    # was there something on the incoming socket that we listen on?
    if sock in incoming:
        # yep, we need to accept the new client
        newsock, addr = sock.accept ()

        # do we have enough clients already?
        if len ( clients ) == 2:
            # oops, we already have 2 clients, we accept no more of 'em, send a message and close
            # the socket
            Connection ( newsock ).send ( 'full\n' )
            newsock.close ()

        else:
            # create and return the new sockets as connections
            newclient = Player ( Connection ( newsock ) )

            # send a welcome message to the client so that he/she knows all is ok
            newclient.getConnection ().send ( 'ok\n' )

            # add the new client to the list of all active clients/players
            clients.append ( newclient )
    
    # return the list of ready clients
    return (ready, clients)


#################################################################################################
def getOtherClient (clients, other):
    """Returns the other client from the list of clients, compared to 'other'. So, you you have one
    client and want the other, but do not want to do an ugly 'if' all the time this method can be
    used. If the passed list contains only one client then None is returned."""

    # loop over all clients
    for client in clients:
        # is this another client?
        if client != other:
            # yep, this is it
            return client

    # no other client found, means the list contain only one client (the compared one), so just
    # return None to indicate that
    return None


#################################################################################################
def loadScenario (scenarioid):
    """Loads the scenario given by 'scenarioname'."""
 
    # load the scenario too
    print "loadScenario: loading scenario: %d" % scenarioid
        
    # loop over all scenario infos we got
    for info in scenario.scenario_manager.getScenarios ():
        # is this the wanted
        if scenarioid == info.getId ():
            # yep, here it is, parse the file
            ScenarioParser ().parseFile ( info.getFile () )
   

#################################################################################################
def startGame (clients, scenarioid):
    """This function starts the game with the passed clients. Mainly sends them the 'start' message
    so that they know that they should start the actual game."""

    # load the scenario
    loadScenario ( scenarioid )

    # loop over all units
    for unit in scenario.info.units.values ():
        # set proper visibility on the unit.
        unit.calcView()
    
    # loop over all clients
    for client in clients:
        # send the message
        client.getConnection ().send ( 'start\n' )
        print "sending 'start' to:", client


#################################################################################################
def initLoop (socket):
    """This function is used by the server when it is initializing the clients. After both the
    clients have connected they will perform various setup tasks, such as choose a scenario, the
    side and other options. This function will inform the players that they may continue with the
    setup procedure, and then read requests from the players until the decide to start the game. It
    then verifies that all data given by the players is sane, and if all is ok returns a list with
    two instances of Player, after which the main game loop can begin. On error 0 is returned.

    The passed 'socket' is a socket ready for listening.
    """

    # a list of instances of Player
    clients = []

    # name of chosen scenario
    chosen = ''
    
    # loop forever
    while 1:
        # wait for data to arrive
        ready, clients = waitForData ( clients, socket )

        # loop over the clients that are ready
        for client in ready:
            
            # attempt to read a line using its connection if one is available. To avoid polling we
            # use a short timeout here
            line = client.getConnection ().readLine ( 0.05 )

            # did we get anything?
            if not line:
                # nope, loop on
                continue
            
            # clean up the line 
            line = strip ( line )
            
            # what did we get? Check all possible strings we should be able to get
            if line == 'get_scenarios':
                # client wants the scenarios, send back a list with info about all senarios we have
                scenario.scenario_manager.sendScenarioInfo ( client.getConnection () )

                
            elif line.split ()[0] == 'start':
                # parse out the data the player chose
                side       = int ( line.split ()[1] )
                scenarioid = int ( line.split ()[2] )

                # assign the data
                client.setSide ( side )

                # get the other client
                other = getOtherClient ( clients, client )

                # did we get anything?
                if other == None or other.getSide () == UNKNOWN:
                    # only one client that has started the game, send the 'ok' to him/her, as the
                    # data is always correct for the first client
                    client.getConnection ().send ( 'ok\n' )

                    # store name of chosen scenario
                    chosen = scenarioid
                    continue

                # was the scenario the same?
                if chosen != scenarioid and chosen != '':
                    # oops, bad senario chosen
                    print "sending bad_scenario '%s' != '%s'" % (chosen, scenarioid)
                    client.getConnection ().send ( 'bad_scenario %d\n' %  scenarioid )
                    continue
                
                # was the same side chosen? The players must have different scenarios
                if other.getSide () == side:
                    # oops, same side chosen, send error to the last client
                    print "sending 'bad_side'"
                    client.getConnection ().send ( 'bad_side\n' )
                    
                else:
                    # the sides are different, all is ok, inform the player that all is ok
                    # parameterwise and then start the game for both players
                    client.getConnection ().send ( 'ok\n' )
                    startGame ( clients, chosen )

                    # go away from the loop and return the list of clients
                    return clients
                    
            elif line.split ()[0] == 'scenario':
                # send the chosen scenario to the client
                scenario.scenario_manager.sendScenario ( client.getConnection (),
                                                         join ( line.split ()[1:] ) )
 

            elif line == 'quit':
                # remove the client from the list of clients. TODO: this is just a temporary solution
                clients.remove ( client )
                print 'init_loop: got quit, removing client'

                
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
