###############################################################################################
# $Id: action.py,v 1.45 2002/09/08 19:18:03 msa Exp $
###############################################################################################

import sys
import copy
import pygame
from pygame.locals import *

import scenario
import properties
import constants

import state
import idle
import menu

import immediate.end_action


class Action (state.State):
    """
    This class is a state that is used for displaying action that has been computed at the
    server. The player can sit and what what has happened during the last action phase, but can't
    really interact in any way.

    This state will update the time and perform regular updates to the internal datastructures as
    the events on the battlefield progress. When the action has been shown then the player can again
    go back and give new orders.

    This state will transfer into the idle state when done. Both the client and server use this state.

    When the action is started for the first time we do a deep copy of scenario.info.units and save
    it. Then we manipulate those units when applying action for each step. If the player never moves
    backward/forward in the time display this is all good and nothing extra needs to be done. If the
    player however moves the time display forward backward some stuff needs to be done:

    * forward: apply all steps up to the new desired step
    * backward: start from the 'clean' deep copy, apply all action starting from the beginning up to
                this new step.
    * goto start: copy back the deep copy.
    
    """

    def __init__ (self):
        """Initializes the instance. Sets default values for all needed member."""

        # call superclass constructor
        state.State.__init__ (self)

        # set default cursor
        self.setDefaultCursor ()
       
        # set defaults
        self.name = "action"

        # now setup the unit we have so that the proper keymap and helptexts are shown
        self.setupKeymapAndHelp ()

        # start from the first step. This is really not necessary, but is here more as a reminder
        scenario.time.firstStep ()

        # reset the flag that indicates that a new turn has begun. this must be rest here, as the
        # other player may just skip through the action and press "end turn" on the orders before
        # we've set the flag otherwise. So, rather too early than too late
        scenario.turn_ended = 0

        # by default we're paused
        self.ispaused = 1

        # if end-of-animation text has been shown
        self.has_showed_end_text = 0

        # create a deep copy of all units. this backup copy is used when we go back in time in the
        # action 
        self.unitscopy = copy.deepcopy ( scenario.info.units )

        # find the action buttons layer
        self.actionlayer = scenario.playfield.getLayer ( "action" )

        # set a pause mode for the layer too, ie. show the play button
        self.actionlayer.setPausedMode ()

        # and make it visible
        scenario.playfield.setVisible ( self.actionlayer )

        # we also want to hide the orders layer if it's visible
        self.ordersview_visible = scenario.playfield.getLayer ( "orders_view" ).isVisible ()

        # was it visible?
        if self.ordersview_visible:
            # yes, hide it
            scenario.playfield.setVisible ( scenario.playfield.getLayer ( "orders_view" ), 0 )
        
        # show a small message to the player
        scenario.messages.add ( 'Starting action for turn %d' % scenario.time.getTurn () )

        # the last applied step isn't yet valid
        self.lastapplied = -1

    
    def setupKeymapAndHelp (self):
        """Sets up the internal keymap and the help texts. This method also creates the menu keymap
        that is sent to the popup menu.""" 
 
        # add the needed stuff to the keymap
        self.keymap [ ( K_p, KMOD_NONE ) ] = self.pause
        self.keymap [ ( K_f, KMOD_NONE ) ] = self.forward
        self.keymap [ ( K_b, KMOD_NONE ) ] = self.backward
        self.keymap [ ( K_a, KMOD_NONE ) ] = self.gotoStart
        self.keymap [ ( K_e, KMOD_ALT  ) ] = self.endAction

        # add to the help texts too
        self.helptext.append ( " " )
        self.helptext.append ( "p -  pause action" )
        self.helptext.append ( "f -  forward 10s" )
        self.helptext.append ( "b -  backward 10s" )
        self.helptext.append ( "a -  to start of action" )

        # set a help topic
        self.help_topic = 'action'

       
    def handleLeftMousePressed (self, event):
        """Handles a click with the left mouse button. Checks wether one of the action buttons has
        been pressed. If that's the case then the proper callback is triggered. It also checks the
        admin menu in the unit info window to see if it has been clicked. If it has the proper
        callback is executed.
        """ 

        # TODO: fix this using some proper mechanism
        import menu

        # find the question layer
        unit_info_layer = scenario.playfield.getLayer ( "info_view" )

        # is the menu button in that layer clicked?
        if unit_info_layer.isMenuButtonClicked ( event.pos ):
            # it's clicked, so activate a state to handle the menu
            key, modifier = menu.Menu ( self.adminmenukeymap, event.pos ).run ()

            # did we get a valid key?
            if key != None:
                # call the proper callback. we assume the modifier is ok
                return self.keymap [ (key, modifier) ] ()


        # get the pos of the click
        x, y = event.pos

        # get all layers from the playfield that are clickable
        clickable = scenario.playfield.getClickableLayers ()

        # loop over all clickable layers and see if any of them wants to handle the click
        for layer in clickable:
            # is it visible? we don't allow invisible layers to be clicked
            if not layer.isVisible ():
                # hidden, so ignore it
                continue
            
            # handle it and see wether it did something with it. if it has handled the click we stop
            # the traversing
            newstate, handled = layer.handleLeftMousePressed ( x, y )

            # did we get a new state?
            if newstate:
                # it handled it and gave us a new state
                return newstate

            # is the layer our action layer, and the event was it handled?
            if layer == self.actionlayer and handled:
                # yes, it's our layer, and it did something. check the button that was pressed
                button = layer.getClickedButton ()

                # a button pressed?
                if button:
                    # yes, call a nice callback. we use the button name to lookup a callback method
                    # and then just call it
                    { 'playpause' : self.pause,
                      'tostart'   : self.gotoStart,
                      'forward'   : self.forward,
                      'rewind'    : self.backward } [button] ()


            # ok, no new state, was it handled anyway?
            if handled:
                # yes, it was handled, we're done
                return

        # the click is on the main playfield, activate a unit if needed
        self.findClickedUnit ( self.toGlobal ( event.pos ) )


    def findClickedUnit (self, (x, y) ):
        """Finds and activates the unit that is in the clicked position (x,y). The position is in
        global coordinates. The main task of the method is to find and make selected any clicked
        units, or clear all selected units if none was clicked."""

        #scenario.messages.add ( 'click in the map at %d %d' % ( x, y ) )
        
        # get the units
        units = scenario.info.units
 
        # loop over all units
        for unit in units.values ():
            # get the unit's position
            (unitx, unity) = unit.getPosition ()
            
            # calculate the distance from the given point to the
            distance = abs (x - unitx) * abs (x - unitx) + abs (y - unity) * abs (y - unity)

            # is the distance within picking range?
            if distance < 100:
               # set it as a selected unit
                self.setSelectedUnit ( unit )

                # we have a new selected unit
                scenario.dispatcher.emit ( 'unit_selected', unit )

                # we're done, go away
                return
        
        # we got this far, no unit was clicked, so clear any selected units if any were shown
        
        # clear selected units
        if self.clearSelectedUnits ():
            # we have no selected unit at all
            scenario.dispatcher.emit ( 'unit_selected', None )
       
       
      
    def handleTimerEvent (self):
        """Event handler for when the the timer fires. Gets the current timer step and applies all
        action for it. Tries to advance to the next step, and if that failes (end of action), then
        the timer events are disabled."""

        # get current step
        step = scenario.time.getStep ()

        # apply action for this step
        self.applyAction ( step )

        # advance the step one further if possible
        if not scenario.time.nextStep ():
            # could not advance any more, stop the timer so that we don't get called unnecessary
            self.disableTimer ()
                    
            # TODO: popup?

            if self.has_showed_end_text == 0:
                self.has_showed_end_text = 1
                scenario.messages.add("End of animation")

    def pause (self):
        """Callback triggered when the player clicks 'pause' or 'play' buttons, or chooses the
        corresponding option in the popup menu. Pauses the execution, and if already paused, starts
        it again.""" 

        # are we paused?
        if self.ispaused:
            # yep, restart the timer
            self.enableTimer ( 1000 )
            
            # not paused now
            self.ispaused = 0

            # set a playing mode for the layer too, ie. show the pause button
            self.actionlayer.setPlayingMode ()
        else:
            # we're pausing, disable all timer events
            self.disableTimer ()


            # we're paused now
            self.ispaused = 1

            # set a pause mode for the layer too, ie. show the play button
            self.actionlayer.setPausedMode ()

        # perform repainting of the playfield so that the button is updated
        scenario.playfield.needInternalRepaint ( self.actionlayer )


    def forward (self):
        """Moves the current step 10 steps forward. This callback is triggered when the player
        clicks the 'forward' button or selects the corresponding menu entry in the popup menu.

        What this callback does is advance the time 10 steps and apply the action for each of the
        'skipped' steps and the repaint the playfield.
        """

        # have we already reached the last step?
        if scenario.time.getStep () == scenario.time.getMaxStep ():
            # yep, max reached, nothing to do here
            return

        # move the current step forward 10 steps
        newstep = scenario.time.getStep () + 10
 
        # too far? we can't have a too large current step
        if newstep > scenario.time.getMaxStep ():
            # use the max step, *and* add 1 so that we actually apply the last step too
            newstep = scenario.time.getMaxStep () + 1

        # now loop over all steps from the last applied step to the new current step to re-apply the
        # action. so if the new step is 27 we loop over all steps from 17 to 27 and apply them
        for step in range ( scenario.time.getStep (), newstep ):
            # apply it
            self.applyAction ( step )

        # set the new step too. this is a little bit ugly as we want to be able to get the last step
        # applied (the +1 above), but not overflow setStep()
        if newstep == scenario.time.getMaxStep () + 1:
            # we're over the max, use the max allowed instead
            scenario.time.setStep ( scenario.time.getMaxStep () )
        else:
            # not yet too far, use the step we calculated
            scenario.time.setStep ( newstep )

        if scenario.time.getStep () == scenario.time.getMaxStep ():
            # show a small message to the player
            if self.has_showed_end_text == 0:
                self.has_showed_end_text = 1
                scenario.messages.add("End of animation")

        # update the selected units
        self.updateSelectedUnits ()
        
        # repaint the playfield so that the new action is shown
        scenario.playfield.needRepaint ()


    def backward (self):
        """Moves the current step 10 steps backward. This callback is triggered when the player
        clicks the 'backward' button  or selects the corresponding menu entry in the popup menu.

        What this callback does is move the current step 10 steps backward. It will then resore the 
        saved original copy of the units (as they were at the start of the action), and apply all
        the steps from 0 to the new current step. We can't simply go back one step, so instead we
        must re-apply the action all the way from the beginning. Finally the method will repaint the
        playfield. 

        TODO: a little bit ugly, are there any better ways of doing this?
        """

        # move the current step back 10 steps
        newstep = scenario.time.getStep () - 10
 
        # too far? we can't have a negative current step
        if newstep < 0:
            newstep = 0

        # do a deep copy and restore the units as they were before the action started. this "undoes"
        # all the action that has been applied so far
        scenario.info.units = copy.deepcopy ( self.unitscopy )

        # update the selected units
        self.updateSelectedUnits ()

        # now loop over all steps from 0 to the new current step to re-apply the action. so if the
        # new step is 27 we loop over all steps from 0 to 27 and apply them
        for step in range ( newstep ):
            # apply it
            self.applyAction ( step )

        # set the new step too
        scenario.time.setStep ( newstep )
       

    def gotoStart (self):
        """ """
        print "Action.start: move the the start of the action"

        # we're pausing now as we move to start, disable all timer events
        self.disableTimer ()

        # we're paused now
        self.ispaused = 1

        # set a pause mode for the layer too, ie. show the play button
        self.actionlayer.setPausedMode ()

        # move to the first step in the timer and make sure that we apply that step next time we
        # apply action data for a step
        scenario.time.firstStep ()
 
        # restore the copy of the units. this contains the units as they were when the action
        # started, as it's a deep copy
        scenario.info.units = copy.deepcopy ( self.unitscopy )

        # reset all selected units too
        self.updateSelectedUnits ()
        
        # repaint the playfield so that the new action is shown
        scenario.playfield.needRepaint ()


    def applyAction (self, step):
        """Applies the action for the current time step. All events that have been given for the
        current action step are updated, then an update of the playfield is forced."""

        # was this the step that was last applied?
        if step == self.lastapplied:
            # yep, don't apply it twice
            print "Action.applyAction: action for %d already applied, ignoring" % step
            return
        
        print "Action.applyAction: applying action for step %d" % step
 
        # set the new step too
        scenario.time.setStep ( step )

        # this is now the last applied step
        self.lastapplied = step
        
        # repaint the playfield so that the new action is shown. we always register a new repaint,
        # which may be a little bit overkill, but rather one paint for a turn where it wasn't
        # needed, than a missed paint
        scenario.playfield.needRepaint ()
       
        # do we have any action for this step?
        if not scenario.events.has_key ( step ):
            # nope
            return
            
        # get all action
        action_for_step = scenario.events [ step ]
        
        # loop over them all
        for act in action_for_step:
            # apply it
            #print "Action.applyAction: executing", str(act).strip ()
            act.execute ()
        

    def endAction (self):
        """Callback triggered when the user presses the 'Alt + e' keys. Asks the player wether
        he/she really wants to end the action phase, to avoid mistakes. If the player confirms the
        ended action state then this state is closed and the Idle state activated. """

        # activate a question state and return it
        if self.askQuestion ( ['Do you really want to end the action phase?' ] ):
            # the answer to our surrender question was accepted, close the state
            return self.close ()
 
        # rejected, so nothing to do here
        return None


    def endTurn (self):
        """Callback triggered when the user chooses 'End action/turn' from the admin popup
        menu. This method is overridden from the one in State so that the action is ended, not the
        turn. """
        # just call our proper method
        return self.endAction ()


    def close (self):
        """Ends the state and hides the dialog and repaints the playfield. Also advances the current
        turn one forward. """

        # no more timer
        self.disableTimer ()

        # advance over all steps for this turn that have not yet been applied. this must be done so
        # that the internal state is up to date when ending the turn
        for step in range ( scenario.time.getStep (), scenario.time.getMaxStep () + 1 ):
            # apply action for this step
            self.applyAction ( step )


        # delete all action that we got for this turn
        scenario.events = {}
        
        # advance turn
        turn = scenario.time.advanceTurn ()

        # have we reached the end of the game?
        if turn >= scenario.info.getMaxTurns ():
            # yep, no longer playing
            print "Action.close: game ended, no more turns"
            scenario.playing = constants.GAME_ENDED

        # has the game ended some other way? the flag 'playing' can have beenset by some action data
        # during the visualization
        if scenario.playing == constants.PLAYING_FINAL_TURN:
            # yep, no longer playing
            print "Action.close: game ended, something else has happened"
            scenario.playing = constants.GAME_ENDED
        
        # make the action layer invisible, not needed anymore
        scenario.playfield.setVisible ( self.actionlayer, 0 )

        # perform backup of the unit's states before we start giving orders. giving orders changes
        # them, and we need a pristine copy of the units
        for unit in scenario.info.units.values ():
            # push the state internally
            unit.pushState ()

        # check for reinforcements
        self.checkReinforcements ()

        # was the window showing the orders visible?
        if self.ordersview_visible:
            # yes, show it again
            scenario.playfield.setVisible ( scenario.playfield.getLayer ( "orders_view" ), 1 )

        # redraw minimap, because units have moved
        scenario.playfield.getLayer("minimap").populateMap()

        # start the action
        return idle.Idle ()


    def checkReinforcements (self):
        """Checks wether any reinforcements have arrived this turn. This is done by iterating the
        global data structure that contains the arrival turns. If any new unit have arrived they
        will be made available and shown."""
        # no reinforcements yet
        new_reinforcements = 0
        
        # get the current turn
        current_turn = scenario.time.getTurn ()

        # loop over all reinforcements that are available
        for company, arrive_turn in scenario.info.reinforcements.values ():
            # has this unit arrived at the battlefield?
            if arrive_turn > current_turn:
                # not yet, nex please
                continue

            # so the unit has arrived, add it to the global structure of available units
            scenario.info.units [ company.getid ()] = company

            # make the unit visible too
            company.setVisible ( 1 )

            # remove the unit from the map
            del scenario.info.reinforcements [ company.getid () ]
            
            # we have something now
            new_reinforcements += 1

        # ok, all units iterated, did we get anything this turn?
        if new_reinforcements > 0:
            # show a message to the player that new reinforcements have arrived on the battlefield
            self.showMessage ( ['General,', 'We have received %d new reinforcements!' % new_reinforcements ] )


    def updateSelectedUnits (self):
        """Updates the selected units. This means that the list of selected units is refreshed,
        i.e. the objects in the list are fetched from the current unit list. This is needed when
        deep copies are done from the main units, and the selected ones are no longer the same
        references. This just loops over the selected units and copies them from the main list to a
        new selection list."""

        # no units yet
        selected = []

        # loop over all units that are selected
        for unit in scenario.selected:
            # does this unit still exist?
            if scenario.info.units.has_key ( unit.getId () ):
                # yep, it's still there, copy it
                selected.append ( scenario.info.units [unit.getId ()] )

        # store the new selection list
        scenario.selected = selected
        
        
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
