###############################################################################################
# $Id: end_orders.py,v 1.12 2002/09/07 09:33:43 msa Exp $
###############################################################################################

import sys
import copy
import pygame
import pygame.event
from pygame.locals       import *

import scenario
import properties

import state
import action
import calculate_action

import immediate.end_turn

class EndOrders (state.State):
    """
    This class is a state that is used when the player has decided to end the orders phase. The
    tasks of this state are to make sure that all orders are sent to the server (or if we're the
    server, that they are put into the proper structures) and that the action is being computed.

    This state will transfer into the action state when done.
    """

    def __init__ (self):
        """Initializes the instance. Sets default values for all needed member."""

        # call superclass constructor
        state.State.__init__ (self)

        # set default cursor
        self.setDefaultCursor ()
       
        # set defaults
        self.name = "end_orders"

        # find the layer that we use for information
        self.layer = scenario.playfield.getLayer ( "end_orders" )

        # and make it visible
        scenario.playfield.setVisible ( self.layer )

        # if we're the server the other client may already have sent us all the plans we needed, so
        # we must somehow check wether we've already got a 'end_turn' immediate plan. if we have one
        # we're ready to start calculating orders.
        self.calculated = 0
        if not scenario.run_as_server:
            # send off the plans
            self.sendPlans ()
        else:
            # running as server, check if we should now start doing action calculations. The reason we
            # don't do it is that we need to return a new state CalculateAction, and we can't do that
            # from a constructor. Instead we post a dummy event and get called real soon
            self.callMeSoon()

        # loop over the units and restore their state to what it was before the player gave them orders
        for unit in scenario.info.units.values ():
            unit.popState ()


    def handleEvent (self, event):
        """This event handler is only useful the first time it is called.
        This is really a little kludge, see comment in the constructor above.
        NOTE: both client and server will call this, although it is only valid
        for the server, and only once."""

        if self.calculated == 0 and scenario.run_as_server and scenario.turn_ended:
            # start doing action calculations
            self.calculated = 1
            return self.calculateAction ()

        # Otherwise, perform the default handling
        return state.State.handleEvent(self, event)


    def handleImmediate (self, update):
        """Event handler that lets a state handle an incoming immediate update. The update has been
        read from the network connection and is usually important. States that can handle updates
        should do it.
        """
        print "Got immediate", update.getName()

        # what update is it?
        if update.getName () == 'endturn':
            # we got an end turn
            print "EndOrders.handleImmediate: ending turn %d" % update.getTurn ()

            # the client has now sent an 'endturn', which means that all plans have been sent to us
            # and it's waiting for some action data. close this state and start crunching data
            return self.calculateAction ()

        elif update.getName () == 'endaction':
            # we got an end of action. this means that the server has now finished calculating the
            # action and sent us all data we need. we're ready to start displaying it
            return self.viewAction ()
        
        else:
            # perform default handling
            state.State.handleImmediate ( self, update )


    def sendPlans (self):
        """Sends all plans that the units have to the server. As we're the client we need to send
        the number of plans as an immediate update: 'end_turn' so that the server knows how many
        plans to expect""" 

        # all plans
        plans = []
        
        # loop over all units in the map
        for unit in scenario.info.units.values ():
            # is the unit visible
            if unit.getOwner () != scenario.local_player_id:
                # not our unit, get next
                continue

            # add the unit plans
            plans += unit.getPlans ()

        # now loop over all plans and send them
        for plan in plans:
            scenario.connection.send ( plan.toString () )
  
        print "EndOrders.sendPlans: sent %d plans" % len (plans)
            
        # first send the number of plans
        self.sendEndTurn ( len (plans) )
        
 
    def sendEndTurn (self, count):
        """Sends an immediate update to the server telling how many plans we'll be sending to it
        very soon. This method should only be executed by the client, never the serever."""
        
        # create a new plan that says how many plans the server should expect
        update = immediate.end_turn.EndTurn ( scenario.time.getTurn () )
        
        # send off the plan
        scenario.connection.send ( update.toString () )

        print "EndOrders.sendEndTurn: sent", update

       
    def calculateAction (self):
        """Ends the state and hides the dialog and repaints the playfield. Starts calculating the
        action in a separate state. All data from the clients should have been received by now."""

        # find the layer and hide it
        scenario.playfield.setVisible ( self.layer, 0 )

        print "EndOrders.sendPlans: about to start CalculateAction state"
 
        # start the action calculateion
        return calculate_action.CalculateAction ()

        
    def viewAction (self):
        """Ends the state and hides the dialog and repaints the playfield. Starts viewing the
        action, which much have been received by now."""

        # find the layer and hide it
        scenario.playfield.setVisible ( self.layer, 0 )

        # start the action
        return action.Action ()


        
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
