###############################################################################################
# $Id: own_unit.py,v 1.89 2002/09/07 10:37:13 msa Exp $
###############################################################################################

import sys
import scenario
import properties
import pygame
from pygame.locals    import *

import state
import idle
import menu
import retreat
import move_unit
import move_unit_fast
import rotate_unit
import check_los
import weapon_info
#import select
import change_combat_policy
import skirmish_unit
import assault_unit

from plan.change_mode         import ChangeMode
from plan.wait                import Wait

from mode.mode_factory        import createMode
from unit                     import Headquarter

import messages

class OwnUnit (state.State):
    """
    This class is a state that provides basic handling of own units. This state should be activated
    when the player clicks on an own unit or some other way makes an own unit active. From this
    state the player can perform various other actions with a unit.

    This state paints all available information about the current unit in the panel.

    This state will activate the following other states:

    * idle if the player clicks in the terrain somewhere.
    * enemy_unit if an enemy unit is clicked.

    * own_unit is kept if another own unit is clicked on.
    """

    def __init__ (self):
        """Initializes the instance. Sets default values for all needed members."""

        # call superclass constructor
        state.State.__init__ (self)
      
        # set default cursor
        self.setDefaultCursor ()

        # set defaults
        self.name = "own_unit"

        # now setup the unit we have so that the proper keymap and helptexts are shown
        self.setupKeymapAndHelp ()


    def setupKeymapAndHelp (self):
        """Sets up the internal keymap and the help text depending on the mode the selected unit is
        in. All types of orders can not be given at all times. This method will check the current
        mode of the unit and the possible modes that can be entered from it and builds up the keymap
        and helptexts from that info.

        This method also creates the menu keymap that is sent to the popup menu."""

        # TODO: we can have many units selected at the same time, so the available commands should
        # be those that all units can perform, otherwise we can end up with illegal orders for some
        # units 

        # get the mode of the current unit
        unit = self.getSelectedUnit ()
        mode = unit.getMode ()
        
        # add the default part of the keymap, i.e the commands that can always be given, no matter
        # what mode the unit is in
        self.keymap [ ( K_i,         KMOD_NONE  ) ] = self.showWeaponInfo
        self.keymap [ ( K_l,         KMOD_NONE  ) ] = self.checkLOS 
        self.keymap [ ( K_h,         KMOD_NONE  ) ] = self.activateHeadquarter
        self.keymap [ ( K_a,         KMOD_SHIFT ) ] = self.activateAll
        self.keymap [ ( K_TAB,       KMOD_NONE  ) ] = self.center
        self.keymap [ ( K_BACKSPACE, KMOD_NONE  ) ] = self.cancel
        self.keymap [ ( K_1,         KMOD_ALT   ) ] = self.toggleAIDebug 

        # setup an initial menu keymap
        self.menukeymap = [ ( "check line of sight", K_l,         KMOD_NONE  ),
                            ( "cancel last order",   K_BACKSPACE, KMOD_NONE  ),
                            ( "find unit HQ",        K_h,         KMOD_NONE  ),
                            ( "activate all",        K_a,         KMOD_SHIFT ),
                            ( "center around unit",  K_TAB,       KMOD_NONE  ),
                            ( "unit weapon unit",    K_i,         KMOD_NONE  ) ]
                           
        
        # define the default part of the help help text too
        self.helptext.append ( " " )

        # can the unit move?
        if mode.canMove () and unit.getFatigue ().checkMove ():
            # yep, moving is possible
            self.keymap [ (K_m, KMOD_NONE) ] = self.move 
            self.menukeymap.append ( ( "move unit", K_m, KMOD_NONE ) ) 
            self.helptext.append ( "m - move unit" )
 
        # can the unit move fast?
        if mode.canMoveFast () and unit.getFatigue ().checkMoveFast ():
            # yep, moving fast is possible
            self.keymap [ (K_f, KMOD_NONE) ] = self.moveFast
            self.menukeymap.append ( ( "move unit fast march", K_f, KMOD_NONE ) ) 
            self.helptext.append ( "f - move unit fast march" )
 
        # can the unit change facing?
        if mode.canRotate () and unit.getFatigue ().checkRotate ():
            # yep, changing facing is possible
            self.keymap [ (K_o, KMOD_NONE) ] = self.rotate
            self.menukeymap.append ( ( "change facing", K_o, KMOD_NONE ) ) 
            self.helptext.append ( "o - change facing" )
 
        # can the unit retreat?
        if mode.canRetreat () and unit.getFatigue ().checkRetreat ():
            # yep, retreating is possible
            self.keymap [ (K_r, KMOD_NONE) ] = self.retreat
            self.menukeymap.append ( ( "retreat", K_r, KMOD_NONE ) ) 
            self.helptext.append ( "r - retreat" )
 
        # can the unit skirmish?
        if mode.canSkirmish () and unit.getFatigue ().checkSkirmish ():
            # yep, skirmishing is possible
            self.keymap [ (K_s, KMOD_NONE) ] = self.skirmish
            self.menukeymap.append ( ( "skirmish", K_s, KMOD_NONE ) ) 
            self.helptext.append ( "s - skirmish" )

        # can the unit assault?
        if mode.canAssault () and unit.getFatigue ().checkAssault ():
            # yep, assaulting is possible
            self.keymap [ (K_x, KMOD_NONE) ] = self.assault
            self.menukeymap.append ( ( "assault", K_x, KMOD_NONE ) ) 
            self.helptext.append ( "x - assault" )

        # can the unit change mode?
        if mode.canChangeMode () and unit.getFatigue ().checkChangeMode ():
            # yep, changing mode is possible
            self.keymap [ (K_c, KMOD_NONE) ] = self.changeMode
            self.menukeymap.append ( ( "change unit mode", K_c, KMOD_NONE ) ) 
            self.helptext.append ( "c - change unit mode" )

        # can the unit change combat policy?
        if mode.canChangePolicy ():
            # yep, changing mode is possible
            self.keymap [ (K_w, KMOD_NONE) ] = self.wait 
            self.menukeymap.append ( ( "wait", K_w, KMOD_NONE ) ) 
            self.helptext.append ( "w - wait 1 minute" )

        # can the unit wait?
        if mode.canWait ():
            # yep, waiting is possible
            self.keymap [ (K_p, KMOD_NONE) ] = self.changeCombatPolicy 
            self.menukeymap.append ( ( "change combat policy", K_p, KMOD_NONE ) ) 
            self.helptext.append ( "p - change combat policy" )

       
        # now add the rest of the help texts that should be last
        self.helptext.append ( "h - get unit hq" )
        self.helptext.append ( "A - activate all units" )
        self.helptext.append ( "i - show weapon info" )
        self.helptext.append ( "l - check unit line of sight" )
        self.helptext.append ( "tab - center map" )
        self.helptext.append ( "del - cancel last order" )


    def cancel (self):
        """Callback triggered when the player presses the 'cancel' key. This attempts to cancel the
        last issued plan to the currently selected units. The last plan is removed and the unit
        modes are updated to whatever becomes current."""
        
        # loop over all selected unit
        for unit in self.getSelectedUnits ():
            # get all the plans for the unit
            plans = unit.getPlans ()

            # any plans at all for the unit?
            if plans == []:
                # no plans, so we can't really cancel anything either
                continue
            
            # get the current plan, i.e. the last one
            currentplan = plans [ len(plans) - 1 ]

            # and the old mode, i.e the mode thate the unit had before that plan got applied
            oldmode = currentplan.getOldMode ()

            # just pop off the most recently added plan. We simply slice off the last plan
            unit.setPlans ( unit.getPlans () [:-1] )

            # set it as the new current mode
            unit.setMode ( oldmode )

        # now the keymap and help stuff needs to be reset
        self.setupKeymapAndHelp ()

        # we have changed some units
        scenario.dispatcher.emit ( 'units_changed', self.getSelectedUnits () )


    def move (self):
        """Callback triggered when the user presses the key 'm' for moving the unit. This callback
        sets the internal state to be 'MoveUnit'."""
        
        # just activate a new state
        return move_unit.MoveUnit () 
    

    def moveFast (self):
        """Callback triggered when the user presses the key 'f' for moving the unit fast. This callback
        sets the internal state to be 'MoveUnitFast'."""

        # just activate a new state
        return move_unit_fast.MoveUnitFast () 


    def retreat (self):
        """Callback triggered when the user presses the key 'r' for retreating the unit. This callback
        sets the internal state to be 'Retreat'."""

        # just activate a new state
        return retreat.Retreat () 


    def rotate (self):
        """Callback triggered when the user presses the key 'o' for rotating the. This callback
        sets the internal state to be 'RotateUnit'."""

        # just activate a new state
        return rotate_unit.RotateUnit () 


    def changeMode (self):
        """Callback triggered when the user presses the key 'c' for changing the unit mode. This callback
        will change the unit mode for all selected units from a move mode to a battle mode. """
        
        # loop over all selected unit
        for unit in self.getSelectedUnits ():
            # can the unit change its mode?
            if not unit.getMode ().canChangeMode () or not unit.getFatigue ().checkChangeMode ():
                # nope, the unit mode or fatigue prohibits it, next unit
                scenario.messages.add ( '%s can not change mode' % unit.getName (), messages.ERROR )
                continue

            # create a new mode for the unit
            mode = createMode ( createMode ( unit.getMode ().onChangeMode () ).onDone () )

            # assign it too
            unit.setMode ( mode )

            # create a new 'change mode' plan
            plan = ChangeMode ( oldmode=unit.getMode (), unitid=unit.getId () )
 
            # add it to the plans for the unit
            unit.getPlans ().append ( plan )
           

        # now the keymap and help stuff needs to be reset
        self.setupKeymapAndHelp ()

        # we have changed some units
        scenario.dispatcher.emit ( 'units_changed', self.getSelectedUnits () )
        
        # no new state
        return None


    def changeCombatPolicy (self):
        """Callback triggered when the user presses the key 'p' for changing the unit combat policy
        mode. This callback will activate the internal state 'ChangeCombatPolicy' if the unit can
        change the policy from its current state, and let it take care of the bussiness."""

        # just activate a new state
        return change_combat_policy.ChangeCombatPolicy () 


    def wait (self):
        """Callback triggered when the user presses the key 'w' for having the unit wait for a
        certain time. A new plan 'wait' is added to all selected units.
        """
        # loop over all selected units
        for unit in self.getSelectedUnits ():
            # create a new 'wait' plan for waiting 1 minute
            plan = Wait ( oldmode=unit.getMode (), delay=1, unitid=unit.getId () )
 
            # add it to the plans for the unit
            unit.getPlans ().append ( plan )
           
            # create a new mode for the unit
            mode = createMode ( createMode ( unit.getMode ().onWait () ).onDone () )

            # assign it too
            unit.setMode ( mode )

        # now the keymap and help stuff needs to be reset
        self.setupKeymapAndHelp ()

        # we have changed some units
        scenario.dispatcher.emit ( 'units_changed', self.getSelectedUnits () )
        
        # no new state
        return None


    def checkLOS (self):
        """Callback triggered when the user presses the key 'l' for checking LOS for the unit. This callback
        sets the internal state to be 'CheckLos'."""

        # just activate a new state
        return check_los.CheckLos () 


    def skirmish (self):
        """Callback triggered when the user presses the key 's' for setting a skirmish target for the
        unit. This callback sets the internal state to be 'SkirmishUnit'."""

        return skirmish_unit.SkirmishUnit ()


    def assault (self):
        """Callback triggered when the user presses the key 'XXX' for setting an assault target for the
        unit. This callback sets the internal state to be 'AssaultUnit'."""

        return assault_unit.AssaultUnit ()

  
    def showWeaponInfo (self):
        """Callback triggered when the user presses the 'i' key. This method will bring up a small
        dialog that shows information about the weapon used by the current unit."""
        
        # create and return a new state showing our labels
        return weapon_info.WeaponInfo ( self.getSelectedUnit ().getWeapon (), self )


    def center (self):
        """Callback triggered when the player presses the key 'TAB' for centering the screen around
        a unit. The playfield will be scrolled so that it is as centered around the unit as
        possible. If the unit is on the edge of the playfield then it is scrolled as far as
        possible. The same unit is kept selected. If multiple units are selected then the scrolling
        is done wrt the main selected unit."""
        # get the selected unit
        selected = self.getSelectedUnit ()

        # get the hex the unit is in 
        xhex, yhex = scenario.map.pointToHex2 (selected.getPosition () )
        
        # and the visible size of the map as well as the offsets
        xoff, yoff = scenario.playfield.getOffset ()
        xvis, yvis = scenario.playfield.getVisibleSize ()

        # we want to center the map by making the offsets for the map be:
        # unit position - half visible size
        xdelta = xvis / 2
        ydelta = yvis / 2

        # and now calculate the new offsets we want to scroll to
        xscrollto = xhex - xdelta
        yscrollto = yhex - ydelta

        # make sure they are not negative, we can't scroll further away than (0,0)
        if xscrollto < 0:
            xscrollto = 0

        if yscrollto < 0:
            yscrollto = 0

        # perform the scrolling and see wether we actually did anything
        if scenario.playfield.setOffset ( xscrollto, yscrollto ):
            # repaint the playfield now that it has changed
            scenario.playfield.needRepaint ()

        # no new state
        return None


    def activateHeadquarter (self):
        """Finds the headquarter for the unit and makes it the current unit. If the unit has no
        headquarter this method will do nothing. It will first find the parent organization and then
        get the headquarter for that."""

        # get the selected unit
        selected = self.getSelectedUnit ()

        # get the parent organization for the current unit
        parent = selected.getParentOrganization ()

        # do we have a parent?
        if parent == None:
            # no parent, show message
            scenario.messages.add ( 'unit has no headquarter', messages.ERROR )

            return None

        # parent organization was ok, get its hq
        hq = parent.getHeadquarter ()

        # is the hq still alive? we can't activate it if it's destroyed
        if hq.isDestroyed ():
            # it's destroyed, what should we do now?
            scenario.messages.add ( 'headquarter is destroyed', messages.ERROR )
            return None
        
        # make it the new current selected unit
        self.setSelectedUnit ( unit=hq, clearfirst=1 )

        # setup the unit we have so that the proper keymap and helptexts are shown 
        self.setupKeymapAndHelp ()

        # and have the map center around the new unit
        self.center ()

        # we have a new selected unit
        scenario.dispatcher.emit ( 'unit_selected', self.getSelectedUnit () )
                                                       

    def activateAll (self):
        """Activates all units that belong to the same organization as the selected unit. This is
        useful mainly for battallions and regiments. Makes the headquarter the current selected
        unit."""
        
        # get the selected unit
        selected = self.getSelectedUnit ()

        # is that a headquarter?
        if isinstance ( selected, Headquarter ):
            # a headquarter, make it the current unit
            self.setSelectedUnit ( unit=selected, clearfirst=1 )

            # get the organization the hq is hq for
            org = selected.getHeadquarterFor ()
        else:
            # a non-hq unit, get its parent organization
            org = selected.getParentOrganization ()

            # make the hq selected
            self.setSelectedUnit ( unit=org.getHeadquarter (), clearfirst=1 )

        # now get the companies for the organization and add them too as selected units
        for tmp in org.getCompanies ():
            # do not clear the old ones
            self.setSelectedUnit ( unit=tmp, clearfirst=0 )

        # setup the unit we have so that the proper keymap and helptexts are shown 
        self.setupKeymapAndHelp ()

        # and have the map center around the new unit
        self.center ()

        # we have a new selected unit
        scenario.dispatcher.emit ( 'unit_selected', self.getSelectedUnit () )

       
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
