###############################################################################################
# $Id: rotate_unit.py,v 1.37 2002/09/04 17:03:55 msa Exp $
###############################################################################################

import sys
import pygame
import pygame.cursors
import scenario
import properties
from pygame.locals        import *

from plan.rotate import Rotate

import state
import own_unit
import messages

class RotateUnit (state.State):
    """
    This class is a state that takes care of letting the user rotate a unit. This is done by waiting
    for a click in the map and then having the unit rotate to face that click. This state should
    only be activated when an own unit is selected. When the user has clicked in the map and the
    rotation has been dispatched to the server the state OwnUnit is activated.

    A packet 'rotate' will also be sent to the server if we're the client or just added to the queue
    of data if we're the server.
    """

    # define a shared base cursor
    cursor = None


    def __init__ (self):
        """Initializes the instance. Sets default values for all needed members."""

        # call superclass constructor
        state.State.__init__ (self )

        # do we have a cursor already loaded? 
        if not RotateUnit.cursor:
            # nope, so load it. First get the filenames
            datafile = properties.state_rotate_cursor_data
            maskfile = properties.state_rotate_cursor_mask

            # now load it
            RotateUnit.cursor = pygame.cursors.load_xbm ( datafile, maskfile )

        # set our own cursor cursor
        pygame.mouse.set_cursor ( *RotateUnit.cursor )
       
        # set defaults
        self.name = "rotate_unit"
        
        # set the keymap too
        self.keymap [ (K_ESCAPE, KMOD_NONE) ] = self.cancel

        # define the help text too
        self.helptext = [ "Rotate a unit",
                          " ",
                          "arrow keys - scroll map",
                          "F1 - show this help text",
                          "F10 - toggle fullscreen mode",
                          "F12 - save a screenshot",
                          "esc - cancel the order" ]
        

    def cancel (self):
        """Callback triggered when the user presses the 'escape' key. Cancels the moving and makes a
        OwnUnit state active again."""
        # return a new state
        return own_unit.OwnUnit ()

        
    def handleLeftMousePressed (self, event):
        """Method for handling a mouse pressed. Checks wether the mouse was clicked in the map or in
        the panel. Only clicks in the map will do anything useful, all other clicks are
        ignore. Accepts the new facing target if a click is made in the map and return a new 'OwnUnit'
        if the unit will be rotated.

        A rotation plan in stored for the unit(s) if the rotation was ok.
        """

        # get event position and unit
        x, y = event.pos

        # get current turn
        turn = scenario.time.getTurn ()

        # the click is on the main playfield, so get the clicked coordinate
        globalx, globaly = self.toGlobal ( (x, y) )

        # If outside map, skip
        (hexx, hexy) = scenario.map.pointToHex2((globalx,globaly))
        if not scenario.map.isInside(hexx, hexy):
            return None

        # loop over all selected unit
        for unit in self.getSelectedUnits ():
            # can the unit rotate?
            if not unit.getMode ().canRotate () or not unit.getFatigue ().checkRotate ():
                # nope, the unit mode or fatigue prohibits it, next unit
                scenario.messages.add ( '%s can not rotate' % unit.getName (), messages.ERROR )
                continue

            # get unit position
            unitx, unity = unit.getPosition ()

            # create a new 'rotate' command. Let the id:s have default value, we don't know them here anyway
            plan = Rotate (  oldmode=unit.getMode (), unitid=unit.getId (), x=globalx, y=globaly)

            # add it to the plans for the unit
            unit.getPlans ().append ( plan )
 
        # we have changed some units
        scenario.dispatcher.emit ( 'units_changed', self.getSelectedUnits () )

        # unit is now moved as far as we are concerned, let the engine sort it all out and resume to
        # having the unit activated
        return own_unit.OwnUnit ()
 
    
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
