###############################################################################################
# $Id: terrain.py,v 1.17 2002/08/25 17:54:36 msa Exp $
###############################################################################################

# a dictionary of all terrain types, indexed by a string which is their name
terrainDict = {}


class Terrain:

    """ The terrain type indicates what kind of terrain is dominant in the part of the map that
    the triangle occupies, such as woods, slope, mud etc. The type is used in various calculations.

    This class also provides easy means to use the terrain in various calculations, as it can return
    a modifier suitable for use in movement, attack and defense value calculations.

    TODO: more info about modifiers when we know how to handle them!

    Every subclass should set the data for the movment speed modifiers list.

    Probably modifer inquiries should take an optional unit/type field?
    """          

    def __init__ (self, type):
        """Initializes the instance. Mainly sets default values for all data so that we don't break
        stuff unnecessarily."""

        # store the type
        self.type = type

        # set the movement speed and fatigue modifiers. These are four values that tell how the
        # terrain affects movement for: infantry, cavalry, artillery and headquarters. these must be
        # overridden by sublcasses! 
        self.movement_speed_mods = ( 1.0, 1.0, 1.0, 1.0 )
        self.movement_fatigue_mods = ( 1.0, 1.0, 1.0, 1.0 )
        
        # set default attack- and defense modifiers
        self.attack_mod  = 1.0
        self.defense_mod = 1.0
        

    def getType (self):
        """Returns the type of terrain for this triangle."""
        return self.type


    def movementSpeedModifier (self, unit):
        """This method returns a modifier for the movement speed for the given unit over this
        terrain. The modifier is a value that modifies the theorethical maximum speed of the unit,
        so all returned values are <=1.0. The speed also depends on the unit type. If the terrain is
        impassable then 0 is returned."""

        # return a speed that depends on the type
        return self.movement_speed_mods [ unit.getType () ]


    def movementFatigueModifier (self, unit):
        """This method returns a modifier for the fatigue the unit gains when moving over the
        terrain. The value is 1.0 if the terrain does not add extra fatigue at all, and >1.0 if the
        terrain is harder to move through. The value 1.0 should be like moving on solid, perfectly
        clear road.

        Different units may gain different amounts of fatigue for different types of terrain."""

        # return a speed that depends on the type
        return self.movement_fatigue_mods [ unit.getType () ]

    
    def defenseModifier (self):
        """Returns the defensive modifiers for this terrain. This value indicates how the terrain
        affects the defensive position. It may either increase or decrese the defensive value. An
        increased value means that the position is good for defense."""
        return self.defense_mod
    
    
    def attackModifier (self):
        """Returns the attack modifier for this terrain. This value indicates how the terrain
        affects the attacker in performing the attack. It may either increase or decrese the
        defensive value. A decreased value means that the position is good for attacking, probably
        very open and allows for easy manouvers."""
        return self.attack_mod


    def visionModifier (self,altitude):
        """ """
        if altitude > 0:
            return 1.0
        return 0 # Terrain is opaque below ground.


###############################################################################################
class GrassTerrain(Terrain):
    """  boring short grass"""

    def __init__ (self):
        """Initializes the instance. Sets needed parameters."""
        Terrain.__init__ ( self, "grassland" )
        
        # set the movement speed and fatigue modifiers
        self.movement_speed_mods = ( 1.0, 1.0, 1.0, 1.0 )
        self.movement_fatigue_mods = ( 1.0, 1.0, 1.0, 1.0 )

        # set attack- and defense modifiers too
        self.attack_mod  = 0.9
        self.defense_mod = 0.6


###############################################################################################
class SandTerrain(Terrain):
    """ sand """

    def __init__ (self):
        """Initializes the instance. Sets needed parameters."""
        Terrain.__init__ ( self, "sand" )
        
        # set the movement speed and fatigue modifiers for (inf,cav,art,hq)
        self.movement_speed_mods = ( 0.7, 0.8, 0.7, 0.7 )
        self.movement_fatigue_mods = ( 1.0, 1.0, 1.0, 1.0 )

        # set attack- and defense modifiers too
        self.attack_mod  = 0.8
        self.defense_mod = 0.5


###############################################################################################
class MudTerrain(Terrain):
    """ mud """

    def __init__ (self):
        """Initializes the instance. Sets needed parameters."""
        Terrain.__init__ ( self, "mud" )
        
        # set the movement speed and fatigue modifiers for (inf,cav,art,hq)
        self.movement_speed_mods = ( 0.5, 0.5, 0.5, 0.5 )
        self.movement_fatigue_mods = ( 1.0, 1.0, 1.0, 1.0 )

        # set attack- and defense modifiers too
        self.attack_mod  = 0.7
        self.defense_mod = 0.5


###############################################################################################
class CrackedMudTerrain(Terrain):
    """Dried up cracked mud."""

    def __init__ (self):
        """Initializes the instance. Sets needed parameters."""
        Terrain.__init__ ( self, "crackedmud" )
        
        # set the movement speed and fatigue modifiers for (inf,cav,art,hq)
        self.movement_speed_mods = ( 0.7, 0.7, 0.7, 0.7 )
        self.movement_fatigue_mods = ( 1.0, 1.0, 1.0, 1.0 )

        # set attack- and defense modifiers too
        self.attack_mod  = 0.8
        self.defense_mod = 0.5


###############################################################################################
class RockTerrain(Terrain):
    """ rocks """

    def __init__ (self):
        """Initializes the instance. Sets needed parameters."""
        Terrain.__init__ ( self, "rocks" )
        
        # set the movement speed and fatigue modifiers for (inf,cav,art,hq)
        self.movement_speed_mods = ( 0.5, 0.5, 0.5, 0.5 )
        self.movement_fatigue_mods = ( 1.0, 1.0, 1.0, 1.0 )

        # set attack- and defense modifiers too
        self.attack_mod  = 0.5
        self.defense_mod = 2.0


    def visionModifier (self,altitude):
        if 0 < altitude < 5:
            return .8
        return Terrain.visionModifier(self,altitude)



###############################################################################################
class WaterTerrain(Terrain):
    """ deep water """

    def __init__ (self):
        """Initializes the instance. Nothing can move through water."""
        Terrain.__init__ ( self, "water" )

        # set the movement speed and fatigue modifiers to impassable
        self.movement_speed_mods = ( 0, 0, 0, 0 )
        self.movement_fatigue_mods = ( 1.0, 1.0, 1.0, 1.0 )

        # set attack- and defense modifiers too. we can't be in water, so these values are really bogus
        self.attack_mod  = 0
        self.defense_mod = 0



###############################################################################################
class WoodsTerrain(Terrain):
    """ deep woods """

    def __init__ (self):
        """Initializes the instance. Sets needed parameters."""
        Terrain.__init__ ( self, "woods" )
        
        # set the movement speed and fatigue modifiers for (inf,cav,art,hq)
        self.movement_speed_mods = ( 0.5, 0.4, 0.2, 0.5 )
        self.movement_fatigue_mods = ( 1.0, 1.0, 1.0, 1.0 )

        # set attack- and defense modifiers too
        self.attack_mod  = 0.8
        self.defense_mod = 1.5


    def visionModifier (self, altitude):
        if 0 < altitude < 10:
            return 0.2
        return Terrain.visionModifier(self,altitude)
 

# ###############################################################################################
# class RiverTerrain(Terrain):
#     """Crossable river, ie. a ford.  Very disadvantagous terrain. """
# 
#     def __init__ (self):
#         """Initializes the instance. Sets needed parameters."""
#         Terrain.__init__ ( self, "river" )
#         
#         # set the movement speed and fatigue modifiers for (inf,cav,art,hq)
#         self.movement_speed_mods = ( 0.2, 0.2, 0.1, 0.2 )
#         self.movement_fatigue_mods = ( 1.0, 1.0, 1.0, 1.0 )
# 
#         # set attack- and defense modifiers too
#         self.attack_mod  = 0.2
#         self.defense_mod = 0.5
# 
# 
# 
# ###############################################################################################
# class RoadTerrain(Terrain):
#     """ road. """
# 
#     def __init__ (self):
#         """Initializes the instance. Sets needed parameters."""
#         Terrain.__init__ ( self, "road" )
#         
#         # set the movement speed and fatigue modifiers for (inf,cav,art,hq)
#         self.movement_speed_mods = ( 1.0, 1.0, 1.0, 1.0 )
#         self.movement_fatigue_mods = ( 1.0, 1.0, 1.0, 1.0 )
# 
#         # set attack- and defense modifiers too
#         self.attack_mod  = 1.0
#         self.defense_mod = 0.3
# 
# 

"""This probably isn't the best way to do this, but I'm too lazy to figure out the smart way right
now."""

terrainDict['g'] = GrassTerrain ()
terrainDict['a'] = WaterTerrain ()
terrainDict['w'] = WoodsTerrain ()
terrainDict['s'] = SandTerrain ()
terrainDict['m'] = MudTerrain ()
terrainDict['o'] = RockTerrain ()
#terrainDict['r'] = RiverTerrain ()
#terrainDict['p'] = RoadTerrain ()
terrainDict['c'] = CrackedMudTerrain ()


##         grass   g
##         water   a
##         woods   w
##         sand    s
##         mud     m
##         rock    o
##         river   r
##         road    p
##         crackedmud c


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
